-- Control Manager

log("  ╠ Control Manager")

love.keyboard.setKeyRepeat(true)

control = {
  keys_pressed = { },
  keys_released = { },
  mouse_pressed = { },
  mouse_released = { },
  mouse_down = { },
  buttons_pressed = { },
  buttons_released = { },
  buttons_down = { },
  cues_pressed = { },
  cues_released = { },
  cues_down = { },
  
  -- Default control scheme
  
  cues = {
    down = {key = "down", button = "dpdown"},
    up = {key = "up", button = "dpup"},
    right = {key = "right", button = "dpright"},
    left = {key = "left", button = "dpleft"},
    
    a = {key = "z", button = "a"},
    b = {key = "x", button = "b"},
    x = {key = "a", button = "x"},
    y = {key = "s", button = "y"},
    l = {key = "q", button = "leftshoulder"},
    r = {key = "w", button = "rightshoulder"},
  }
}

function control:update(dt)
  -- Clear controls
  
  self.cues_pressed = { }
  self.cues_released = { }
  self.keys_pressed = { }
  self.keys_released = { }
  self.mouse_pressed = { }
  self.mouse_released = { }
  self.buttons_pressed = { }
  self.buttons_released = { }
  
  -- Handle gamepad retriggering
  
  if self.buttons_latest and self:buttonDown(self.button_latest) then
    self.button_retrigger = self.button_retrigger - dt
    
    if self.button_retrigger <= 0 then
      self.button_retrigger = .025
      love.gamepadpressed(1, self.button_latest)
    end
  else
    self.button_retrigger = .5
  end
end

function control:isDown(cue)
  return self.cues_down[cue]
end

function control:wasPressed(cue)
  return self.cues_pressed[cue]
end

function control:wasReleased(cue)
  return self.cues_released[cue]
end

function control:keyPressed(key)
  return self.keys_pressed[key]
end

function control:keyReleased(key)
  return self.keys_released[key]
end

function control:mousePressed(button)
  return self.mouse_pressed[button]
end

function control:mouseReleased(button)
  return self.mouse_released[button]
end

function control:mouseDown(button)
  return self.mouse_down[button]
end

function control:buttonPressed(button)
  return self.buttons_pressed[button]
end

function control:buttonReleased(button)
  return self.buttons_released[button]
end

function control:buttonDown(button)
  return self.buttons_down[button]
end

function control:direction()
  if self:isDown("down") then
    return "down"
  elseif self:isDown("up") then
    return "up"
  elseif self:isDown("right") then
    return "right"
  elseif self:isDown("left") then
    return "left"
  end
end

function control:directionWasPressed()
  if self:wasPressed("down") then
    return "down"
  elseif self:wasPressed("up") then
    return "up"
  elseif self:wasPressed("right") then
    return "right"
  elseif self:wasPressed("left") then
    return "left"
  end
end

--[[ Bind to LÖVE callbacks

  These have been set to allow the scene, if needs be, to take direct commands.
  Outside of text input, these should seldom be used by scenes.

]]

function love.textinput(text)
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.textInput and scene.text_input_mode then
    scene:textInput(text)
  end
end

function love.keypressed(key, scancode, is_repeat)
  control.keys_pressed[key] = true
  
  local scene = scene_manager:getCurrentScene()
  
  if scene then
    if (not scene.text_input_mode) then
      for i, c in pairs(control.cues) do
        if c.key and c.key == key then
          control.cues_pressed[i] = true
          control.cues_down[i] = true
        end
      end
    end
    
    if scene.keyPressed then
      scene:keyPressed(key, scancode, is_repeat)
    end
  end
end

function love.keyreleased(key, scancode)
  control.keys_released[key] = true
  
  local scene = scene_manager:getCurrentScene()
  
  if scene then
    if (not scene.text_input_mode) then
      for i, c in pairs(control.cues) do
        if c.key and c.key == key then
          control.cues_released[i] = true
          control.cues_down[i] = false
        end
      end
    end
    
    if scene.keyReleased then
      scene:keyReleased(key, scancode)
    end
  end
end

function love.mousemoved(x, y, dx, dy, is_touch)
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.mouseMoved then
    scene:mouseMoved(x, y, dx, dy, is_touch)
  end
end

function love.mousepressed(x, y, button, is_touch, presses)
  control.mouse_pressed[button] = {x = x, y = y, is_touch = is_touch, presses = presses}
  control.mouse_down[button] = true
  
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.mousePressed then
    scene:mousePressed(x, y, button, is_touch, presses)
  end
end

function love.mousereleased(x, y, button, is_touch, presses)
  control.mouse_released[button] = {x = x, y = y, is_touch = is_touch, presses = presses}
  control.mouse_down[button] = false
  
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.mouseReleased then
    scene:mouseReleased(x, y, button, is_touch, presses)
  end
end

function love.wheelmoved(x, y)
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.wheelMoved then
    scene:wheelMoved(x, y)
  end
end

function love.touchmoved(id, x, y, dx, dy, pressure)
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.touchMoved then
    scene:touchMoved(id, x, y, dx, dy, pressure)
  end
end

function love.touchpressed(id, x, y, dx, dy, pressure)
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.touchPressed then
    scene:touchPressed(id, x, y, dx, dy, pressure)
  end
end

function love.touchreleased(id, x, y, dx, dy, pressure)
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.touchReleased then
    scene:touchReleased(id, x, y, dx, dy, pressure)
  end
end

function love.gamepadaxis(joystick, axis, value)
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.gamepadAxis then
    scene:gamepadAxis(joystick, axis, value)
  end
end

function love.gamepadpressed(joystick, button)
  if control.button_latest == nil or control.button_latest ~= button then
    control.button_latest = button
    control.button_retrigger = .5
  end
  
  control.buttons_pressed[button] = true
  control.buttons_down[button] = true
  
  for i, c in pairs(control.cues) do
    if c.button and c.button == button then
      control.cues_pressed[i] = true
      control.cues_down[i] = true
    end
  end
  
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.gamepadPressed then
    scene:gamepadPressed(joystick, button)
  end
end

function love.gamepadreleased(joystick, button)
  if control.button_latest == button then
    control.button_latest = nil
  end
  
  control.buttons_released[button] = true
  control.buttons_down[button] = false
  
  for i, c in pairs(control.cues) do
    if c.button and c.button == button then
      control.cues_released[i] = true
      control.cues_down[i] = false
    end
  end
  
  local scene = scene_manager:getCurrentScene()
  
  if scene and scene.gamepadReleased then
    scene:gamepadReleased(joystick, button)
  end
end