-- Easing functions

log("  ╠ Easing Functions")

function easeInSine(x)
  return 1 - math.cos((x * math.pi) / 2)
end

function easeOutSine(x)
  return math.sin((x * math.pi) / 2)
end

function easeInOutSine(x)
  return -(math.cos(math.pi * x) - 1) / 2
end

function easeIn(x, c)
  return math.pow(x, c)
end

function easeOut(x, c)
  return 1 - math.pow(1 - x, c)
end

function easeInOut(x, c)
  if x < .5 then
    return math.pow(2, (c - 1)) * math.pow(x, c)
  else
    return 1 - math.pow(-2 * x + 2, c) / 2
  end
end

function easeInExpo(x)
  if x == 0 then
    return 0
  else
    return math.pow(2, 10 * x - 10)
  end
end

function easeOutExpo(x)
  if x == 1 then
    return 1
  else
    return 1 - math.pow(2, -10 * x)
  end
end

function easeInOutExpo(x)
  if x == 0 or x == 1 then
    return x
  elseif x < .5 then
    return math.pow(2, 20 * x - 10) / 2
  else
    return (2 - math.pow(2, -20 * x + 10)) / 2
  end
end

function easeInCirc(x)
  return 1 - math.sqrt(1 - math.pow(x, 2))
end

function easeOutCirc(x)
  return math.sqrt(1 - math.pow(x - 1, 2))
end

function easeInOutCirc(x)
  if x < .5 then
    return (1 - math.sqrt(1 - math.pow(2 * x, 2))) / 2
  else
    return (math.sqrt(1 - math.pow(-2 * x + 2, 2)) + 1) / 2
  end
end

local c1 = 1.70158
local c2 = c1 * 1.525
local c3 = c1 + 1

function easeInBack(x)
  return c3 * x * x * x - c1 * x * x
end

function easeOutBack(x)
  return 1 + c3 * math.pow(x - 1, 3) + c1 * math.pow(x - 1, 2)
end

function easeInOutBack(x)
  if x < .5 then
    return (math.pow(2 * x, 2) * ((c2 + 1) * 2 * x - c2)) / 2
  else
    return (math.pow(2 * x - 2, 2) * ((c2 + 1) * (x * 2 - 2) + c2) + 2) / 2
  end
end

local c4 = (2 * math.pi) / 3
local c5 = (2 * math.pi) / 4.5

function easeInElastic(x)
  if x == 0 or x == 1 then
    return x
  else
    return -math.pow(2, 10 * x - 10) * math.sin((x * 10 - 10.75) * c4)
  end
end

function easeOutElastic(x)
  if x == 0 or x == 1 then
    return x
  else
    return math.pow(2, -10 * x) * math.sin((x * 10 - .75) * c4) + 1
  end
end

function easeInOutElastic(x)
  if x == 0 or x == 1 then
    return x
  elseif x < .5 then
    return -(math.pow(2, 20 * x - 10) * math.sin((20 * x - 11.125) * c5)) / 2
  else
    return (math.pow(2, -20 * x + 10) * math.sin((20 * x - 11.125) * c5)) / 2 + 1
  end
end

local n1 = 7.5625
local d1 = 2.75

function easeInBounce(x)
  return 1 - easeOutBounce(1 - x)
end

function easeOutBounce(x)
  if (x < 1 / d1) then
    return n1 * x * x
  elseif (x < 2 / d1) then
    x = x - (1.5 / d1)
    return n1 * x * x + .75
  elseif (x < 2.5 / d1) then
    x = x - (2.25 / d1)
    return n1 * x * x + 0.9375
  else
    x = x - (2.625 / d1)
    return n1 * x * x + 0.984375
  end
end

function easeInOutBounce(x)
  if x < .5 then
    return (1 - easeOutBounce(1 - 2 * x)) / 2
  else
    return (1 + easeOutBounce(2 * x - 1)) / 2
  end
end

-- Lerp

function lerp(x, y, xt, yt, speed)
  local a = xt - x
  
  if yt == nil then
    a = y - x
    local d = xt * love.timer.getDelta()
    if a > 0 then
      return math.min(x + d, y)
    else
      return math.max(x - d, y)
    end
  end
  
  local b = yt - y
  local d = speed * love.timer.getDelta()
  
  if a ~= 0 and b == 0 then
    if a > 0 then
      return math.min(x + d, xt), y
    else
      return math.max(x - d, xt), y
    end
  elseif a == 0 and b ~= 0 then
    if b > 0 then
      return x, math.min(y + d, yt)
    else
      return x, math.max(y - d, yt)
    end
  end
  
  local c = math.sqrt((a * a) + (b * b))
  local a2c = a / c
  local b2c = b / c
  
  local retx, rety = x, y
  if a > 0 then
    retx = x + (a2c * d)
    retx = math.min(retx, xt)
  elseif a < 0 then
    retx = x + (a2c * d)
    retx = math.max(retx, xt)
  end
  if b > 0 then
    rety = y + (b2c * d)
    rety = math.min(rety, yt)
  elseif b < 0 then
    rety = y + (b2c * d)
    rety = math.max(rety, yt)
  end
  
  return retx, rety
end