-- Scene Manager

log("  ╠ Scene Manager")

scene_manager = {
  stack = { },
  just_ended_scene = false
}

function scene_manager:newScene(scene_file, opt, clear_stack)
  local new_scene, error_msg
  if system.cart and love.filesystem.getInfo("active_cart/scenes/" .. scene_file .. ".lua") then
    new_scene, error_msg = love.filesystem.load("active_cart/scenes/" .. scene_file .. ".lua")
  else
    new_scene, error_msg = love.filesystem.load("scenes/" .. scene_file .. ".lua")
  end
  
  if error_msg then
    err(error_msg)
  else
    if self:getCurrentScene() then
      if self:getCurrentScene().no_suspension then
        self:endScene()
      else
        if self:getCurrentScene().onSuspend then
          self:getCurrentScene():onSuspend()
        end
        
        self:getCurrentScene().suspended = true
      end
    end
    
    if clear_stack then
      self:clearStack()
    end
    
    table.insert(self.stack, new_scene())
    
    self:getCurrentScene().sm_filename = scene_file
    self:getCurrentScene().sm_opt = opt
    
    self:getCurrentScene():load(opt)
  end
end

function scene_manager:endScene(depth)
  if not depth then
    depth = 1
  end
  
  if (not self.just_ended_scene) then
    while depth > 0 do
      if self:getCurrentScene().close then
        self:getCurrentScene():close()
      end
      table.remove(self.stack)
      depth = depth - 1
    end
    
    self.just_ended_scene = true
  end
end

function scene_manager:reloadScene()
  if self:getCurrentScene() then
    self:getCurrentScene().no_suspension = true
    self:newScene(self:getCurrentScene().sm_filename, self:getCurrentScene().sm_opt)
  end
end

function scene_manager:clearStack()
  while #self.stack > 0 do
    self:endScene()
    self.just_ended_scene = false
  end
end

function scene_manager:update(dt)
  if self:getCurrentScene() then
    if self:getCurrentScene().suspended then
      if self:getCurrentScene().onResume then
        self:getCurrentScene():onResume()
      end
      
      --log("RESUME")
      
      self:getCurrentScene().suspended = false
    end
    
    self:getCurrentScene():update(dt)
  end
end

function scene_manager:draw()
  if self:getCurrentScene() and (not self:getCurrentScene().system_canvas) then
    self:getCurrentScene():draw()
  end
end

function scene_manager:drawSystem()
  if self:getCurrentScene() and self:getCurrentScene().system_canvas then
    self:getCurrentScene():draw()
  end
end

function scene_manager:getCurrentScene()
  return self.stack[#self.stack]
end