-- Sound Effects Manager

log("  ╠ Audio - SFX Manager")

audio_sfx = {
  active_sources = { }
}

function audio_sfx:init()
  log("Initializing SFX audio...")
  
  self.library = { }
  local total_sfx = 0
  
  for i, sfx in ipairs(love.filesystem.getDirectoryItems("audio/sfx")) do
    local ext = string.sub(sfx, -4)
    local name = string.sub(sfx, 1, -5)
    
    if ext == ".wav" or ext == ".mp3" or ext == ".ogg" then
      local new_sound = love.audio.newSource("audio/sfx/" .. sfx, "static")
      
      self.library[name] = new_sound
      
      total_sfx = total_sfx + 1
    end
  end
  
  -- Create human-readable, alphabetized list
  
  self.list = table.ofKeys(self.library)
  table.sort(self.list)
  
  log("Loaded " .. total_sfx .. " sounds!")
end

function audio_sfx:update(dt)
  for i, sfx in pairs(self.library) do
    sfx:setVolume(settings.volume_sfx * ((system.glitching and .5) or 1) * settings.volume)
    sfx:setEffect("chorus", system.glitching)
    sfx:setEffect("distort", system.glitching)
  end
end

function audio_sfx:play(sound, random_pitch, random_pitch_max)
  if self.library[sound] then
    self.library[sound]:stop()
    
    if random_pitch then
      if type(random_pitch) == "boolean" then
        self.library[sound]:setPitch(1 + (randomPosNeg() * random() % .16))
      elseif random_pitch_max then
        self.library[sound]:setPitch(random(random_pitch * 100, random_pitch_max * 100) / 100)
      else
        self.library[sound]:setPitch(1 + (randomPosNeg() * random() % random_pitch))
      end
    else
      self.library[sound]:setPitch(1)
    end
    
    self.library[sound]:play()
  end
end

function audio_sfx:stop(sound)
  if self.library[sound] then
    self.library[sound]:stop()
  end
end

function audio_sfx:stopAll()
  for i, sfx in pairs(self.library) do
    sfx:stop()
  end
end