-- Stinger Audio Manager

log("  ╠ Audio - Stinger Manager")

audio_stinger = { }

function audio_stinger:init()
  log("Initializing stinger audio...")
  
  local total_stingers = 0
  
  self.stingers = { }
  
  for i, s in ipairs(love.filesystem.getDirectoryItems("audio/stingers")) do
    local new_stinger = "audio/stingers/" .. s
    
    self.stingers[s] = new_stinger
    
    total_stingers = total_stingers + 1
  end
  
  -- Create human-readable, alphabetized list
  
  self.list = table.ofKeys(self.stingers)
  table.sort(self.list)
  
  log("Loaded " .. total_stingers .. " stingers!")
end

function audio_stinger:update(dt)
  if self.current_source then
    self.current_source:setVolume(settings.volume_bgm * ((system.glitching and .5) or 1) * settings.volume)
    self.current_source:setEffect("chorus", system.glitching)
    self.current_source:setEffect("distort", system.glitching)
    
    if not self.current_source:isPlaying() then
      self:stop()
    end
  end
end

function audio_stinger:play(stinger)
  self:stop()
  
  if self.stingers[stinger] then
    self.current_source = love.audio.newSource(self.stingers[stinger], "static")
    self.current_source:play()
    
    self.old_bgm_fade_to = audio_bgm.fade_to + 0
    
    audio_bgm.fade = 0.01
    audio_bgm.fade_to = 0.01
  end
end

function audio_stinger:stop()
  if self.current_source then
    self.current_source:stop()
  end
  
  self.current_source = nil
  
  if self.old_bgm_fade_to then
    audio_bgm:fadeTo(self.old_bgm_fade_to)
    self.old_bgm_fade_to = nil
  end
end