-- Localized Text Manager

log("  ╠ Localized Text Manager")

local localize_cache = { }

function getLocalizedText(file, key, lang_override)
  local lang = settings.lang .. ""
  
  if lang_override then
    lang = lang_override
  end
  
  if (not love.filesystem.getInfo("localization/" .. file .. "_" .. lang .. ".lua")) then
    warn("No localization file for \"" .. file .. "\" in the \"" .. lang .. "\" language.")
    lang = "en"
  end
  
  if localize_cache[file .. "_" .. lang] then
    if type(localize_cache[file .. "_" .. lang]) == "table" then
      return localize_cache[file .. "_" .. lang][key]
    else
      return localize_cache[file .. "_" .. lang]
    end
  end
  
  local success, chunk = pcall(love.filesystem.load, "localization/" .. file .. "_" .. lang .. ".lua")
  if success then
    local success2, text_table = pcall(chunk)
    if success2 then
      localize_cache[file .. "_" .. lang] = text_table
      if text_table[key] then
        return text_table[key]
      else
        err("Localization: key \"" .. key .. "\" not found in file.")
        return "ERR_LOCALIZED_KEY"
      end
    else
      err("Localization: Error calling chunk [" .. text_table .. "].")
      localize_cache[file .. "_" .. lang] = "ERR_LOCALIZED_FILE_CHUNK"
      return "ERR_LOCALIZED_FILE_CHUNK"
    end
  else
    err("Localization: Error calling file \"" .. file .. "\". [" .. chunk .. "].")
    localize_cache[file .. "_" .. lang] = "ERR_LOCALIZED_FILE_CALL"
    return "ERR_LOCALIZED_FILE_CALL"
  end
end

function localize(...)
  return getLocalizedText(...)
end