sepia_shader = [[
  vec4 effect( vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords )
	{
		vec4 c = Texel(tex, texture_coords);
    color.r = min(255.0, (0.393 * c.r) + (0.769 * c.g) + (0.189 * c.b));
    color.g = min(255.0, (0.349 * c.r) + (0.686 * c.g) + (0.168 * c.b));
    color.b = min(255.0, (0.272 * c.r) + (0.534 * c.g) + (0.131 * c.b));
		color.a = c.a; //preserve alpha channel
		return color;
	}
]]

inversion_shader = [[
  vec4 effect( vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords )
	{
		vec4 c = Texel(tex, texture_coords);
    color.r = abs(c.r - 1.0);
		color.g = abs(c.g - 1.0);
		color.b = abs(c.b - 1.0);
		color.a = c.a; //preserve alpha channel
		return color;
	}
]]

glitch_shader = [[
	uniform float millis;
  uniform float strength_1;
  uniform bool glitch;
  uniform bool sepia_tone;
  uniform bool invert;

	//
	// Description : Array and textureless GLSL 2D simplex noise function.
	//      Author : Ian McEwan, Ashima Arts.
	//  Maintainer : stegu
	//     Lastmod : 20110822 (ijm)
	//     License : Copyright (C) 2011 Ashima Arts. All rights reserved.
	//               Distributed under the MIT License. See LICENSE file.
	//               https://github.com/ashima/webgl-noise
	//               https://github.com/stegu/webgl-noise
	//

	vec3 mod289(vec3 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
	}

	vec2 mod289(vec2 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
	}

	vec3 permute(vec3 x) {
    return mod289(((x*34.0)+1.0)*x);
	}

	float snoise(vec2 v)
	{
    const vec4 C = vec4(0.211324865405187,  // (3.0-sqrt(3.0))/6.0
              0.366025403784439,  // 0.5*(sqrt(3.0)-1.0)
              -0.577350269189626,  // -1.0 + 2.0 * C.x
              0.024390243902439); // 1.0 / 41.0
    // First corner
    vec2 i  = floor(v + dot(v, C.yy) );
    vec2 x0 = v -   i + dot(i, C.xx);

    // Other corners
    vec2 i1;
    //i1.x = step( x0.y, x0.x ); // x0.x > x0.y ? 1.0 : 0.0
    //i1.y = 1.0 - i1.x;
    i1 = (x0.x > x0.y) ? vec2(1.0, 0.0) : vec2(0.0, 1.0);
    // x0 = x0 - 0.0 + 0.0 * C.xx ;
    // x1 = x0 - i1 + 1.0 * C.xx ;
    // x2 = x0 - 1.0 + 2.0 * C.xx ;
    vec4 x12 = x0.xyxy + C.xxzz;
    x12.xy -= i1;

    // Permutations
    i = mod289(i); // Avoid truncation effects in permutation
    vec3 p = permute( permute( i.y + vec3(0.0, i1.y, 1.0 ))
        + i.x + vec3(0.0, i1.x, 1.0 ));

    vec3 m = max(0.5 - vec3(dot(x0,x0), dot(x12.xy,x12.xy), dot(x12.zw,x12.zw)), 0.0);
    m = m*m ;
    m = m*m ;

    // Gradients: 41 points uniformly over a line, mapped onto a diamond.
    // The ring size 17*17 = 289 is close to a multiple of 41 (41*7 = 287)

    vec3 x = 2.0 * fract(p * C.www) - 1.0;
    vec3 h = abs(x) - 0.5;
    vec3 ox = floor(x + 0.5);
    vec3 a0 = x - ox;

    // Normalise gradients implicitly by scaling m
    // Approximation of: m *= inversesqrt( a0*a0 + h*h );
    m *= 1.79284291400159 - 0.85373472095314 * ( a0*a0 + h*h );

    // Compute final noise value at P
    vec3 g;
    g.x  = a0.x  * x0.x  + h.x  * x0.y;
    g.yz = a0.yz * x12.xz + h.yz * x12.yw;
    return 130.0 * dot(m, g);
	}

	float rand(vec2 co)
	{
    return fract(sin(dot(co.xy,vec2(12.9898,78.233))) * 43758.5453);
	}

	vec4 effect( vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords )
	{
		vec2 uv = texture_coords;
    vec4 c = Texel(tex, uv);
		float time = millis * 2.0;
    
    if (glitch)
    {
      // Create large, incidental noise waves
      float noise = max(0.0, snoise(vec2(time, uv.y * 0.3)) - 0.3) * (1.0 / 0.7);

      // Offset by smaller, constant noise waves
      noise = noise + (snoise(vec2(time*10.0, uv.y * 2.4)) - 0.5) * 0.15;

      // Apply the noise as x displacement for every line
      float xpos = uv.x - noise * noise * strength_1; //strength_1 used to be 0.25
      c = Texel(tex, vec2(xpos, uv.y));

      // Mix in some random interference for lines
      color.rgb = mix(c.rgb, vec3(rand(vec2(uv.y * time))), noise * 0.3).rgb;
      
      // Apply a line pattern every 4 pixels
      if (floor(mod(texture_coords.y * 0.25, 2.0)) == 0.0)
      {
        color.rgb *= 1.0 - (0.15 * noise);
      }

      // Shift green/blue channels (using the red channel)
      color.g = mix(color.r, Texel(tex, vec2(xpos + noise * 0.05, uv.y)).g, 0.5);
      color.b = mix(color.r, Texel(tex, vec2(xpos - noise * 0.05, uv.y)).b, 0.5);
    }
    
    if (sepia_tone)
    {
      if (glitch)
      {
        color.r = min(255.0, (0.393 * color.r) + (0.769 * color.g) + (0.189 * color.b));
        color.g = min(255.0, (0.349 * color.r) + (0.686 * color.g) + (0.168 * color.b));
        color.b = min(255.0, (0.272 * color.r) + (0.534 * color.g) + (0.131 * color.b));
      } else {
        color.r = min(255.0, (0.393 * c.r) + (0.769 * c.g) + (0.189 * c.b));
        color.g = min(255.0, (0.349 * c.r) + (0.686 * c.g) + (0.168 * c.b));
        color.b = min(255.0, (0.272 * c.r) + (0.534 * c.g) + (0.131 * c.b));
      }
    }
    
    if (invert)
    {
      if (glitch)
      {
        color.r = abs(color.r - 1.0);
        color.g = abs(color.g - 1.0);
        color.b = abs(color.b - 1.0);
      } else {
        color.r = abs(c.r - 1.0);
        color.g = abs(c.g - 1.0);
        color.b = abs(c.b - 1.0);
      }
    }
    
		color.a = c.a; //preserve alpha channel
		return color;
	}
]]

glitch_inverted_shader = [[
	uniform float millis;
  uniform float strength_1;

	//
	// Description : Array and textureless GLSL 2D simplex noise function.
	//      Author : Ian McEwan, Ashima Arts.
	//  Maintainer : stegu
	//     Lastmod : 20110822 (ijm)
	//     License : Copyright (C) 2011 Ashima Arts. All rights reserved.
	//               Distributed under the MIT License. See LICENSE file.
	//               https://github.com/ashima/webgl-noise
	//               https://github.com/stegu/webgl-noise
	//

	vec3 mod289(vec3 x) {
	return x - floor(x * (1.0 / 289.0)) * 289.0;
	}

	vec2 mod289(vec2 x) {
	return x - floor(x * (1.0 / 289.0)) * 289.0;
	}

	vec3 permute(vec3 x) {
	return mod289(((x*34.0)+1.0)*x);
	}

	float snoise(vec2 v)
	{
	const vec4 C = vec4(0.211324865405187,  // (3.0-sqrt(3.0))/6.0
						0.366025403784439,  // 0.5*(sqrt(3.0)-1.0)
						-0.577350269189626,  // -1.0 + 2.0 * C.x
						0.024390243902439); // 1.0 / 41.0
	// First corner
	vec2 i  = floor(v + dot(v, C.yy) );
	vec2 x0 = v -   i + dot(i, C.xx);

	// Other corners
	vec2 i1;
	//i1.x = step( x0.y, x0.x ); // x0.x > x0.y ? 1.0 : 0.0
	//i1.y = 1.0 - i1.x;
	i1 = (x0.x > x0.y) ? vec2(1.0, 0.0) : vec2(0.0, 1.0);
	// x0 = x0 - 0.0 + 0.0 * C.xx ;
	// x1 = x0 - i1 + 1.0 * C.xx ;
	// x2 = x0 - 1.0 + 2.0 * C.xx ;
	vec4 x12 = x0.xyxy + C.xxzz;
	x12.xy -= i1;

	// Permutations
	i = mod289(i); // Avoid truncation effects in permutation
	vec3 p = permute( permute( i.y + vec3(0.0, i1.y, 1.0 ))
			+ i.x + vec3(0.0, i1.x, 1.0 ));

	vec3 m = max(0.5 - vec3(dot(x0,x0), dot(x12.xy,x12.xy), dot(x12.zw,x12.zw)), 0.0);
	m = m*m ;
	m = m*m ;

	// Gradients: 41 points uniformly over a line, mapped onto a diamond.
	// The ring size 17*17 = 289 is close to a multiple of 41 (41*7 = 287)

	vec3 x = 2.0 * fract(p * C.www) - 1.0;
	vec3 h = abs(x) - 0.5;
	vec3 ox = floor(x + 0.5);
	vec3 a0 = x - ox;

	// Normalise gradients implicitly by scaling m
	// Approximation of: m *= inversesqrt( a0*a0 + h*h );
	m *= 1.79284291400159 - 0.85373472095314 * ( a0*a0 + h*h );

	// Compute final noise value at P
	vec3 g;
	g.x  = a0.x  * x0.x  + h.x  * x0.y;
	g.yz = a0.yz * x12.xz + h.yz * x12.yw;
	return 130.0 * dot(m, g);
	}

	float rand(vec2 co)
	{
	return fract(sin(dot(co.xy,vec2(12.9898,78.233))) * 43758.5453);
	}

	vec4 effect( vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords )
	{
		vec2 uv = texture_coords;
		float time = millis * 2.0;

		// Create large, incidental noise waves
		float noise = max(0.0, snoise(vec2(time, uv.y * 0.3)) - 0.3) * (1.0 / 0.7);

		// Offset by smaller, constant noise waves
		noise = noise + (snoise(vec2(time*10.0, uv.y * 2.4)) - 0.5) * 0.15;

		// Apply the noise as x displacement for every line
		float xpos = uv.x - noise * noise * strength_1; //strength_1 used to be 0.25
		vec4 c = Texel(tex, vec2(xpos, uv.y));

		// Mix in some random interference for lines
		color.rgb = mix(c.rgb, vec3(rand(vec2(uv.y * time))), noise * 0.3).rgb;

		// Apply a line pattern every 4 pixels
		if (floor(mod(texture_coords.y * 0.25, 2.0)) == 0.0)
		{
			color.rgb *= 1.0 - (0.15 * noise);
		}

		// Shift green/blue channels (using the red channel)
		color.g = mix(color.r, Texel(tex, vec2(xpos + noise * 0.05, uv.y)).g, 0.5);
		color.b = mix(color.r, Texel(tex, vec2(xpos - noise * 0.05, uv.y)).b, 0.5);
    
    color.r = abs(color.r - 1.0);
    color.g = abs(color.g - 1.0);
    color.b = abs(color.b - 1.0);
    
		color.a = c.a; //preserve alpha channel
		return color;
	}
]]

-- Kudos to Tesselode's wonderful tutorial and examples
-- https://medium.com/@tesselode/using-shaders-to-make-trippy-backgrounds-c109989ea037

battleback_shader = [[
  uniform float amount;
  uniform float density;
  uniform float speed;
  uniform bool interlace;
  uniform float time;
  
  vec4 effect(vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords)
  {
      float offset = amount * sin(texture_coords.y * density + speed * time);
      if (interlace && mod(screen_coords.y, 2.) > 1.) offset *= -1.;
      offset = sqrt(offset);
      texture_coords.y += offset;
      texture_coords.y = mod(texture_coords.y, 1.);
      vec4 texturecolor = Texel(tex, texture_coords);
      return texturecolor * color;
  }
]]

-- Kudos to Micah "Germanunkol" for the tutorial/code on this
-- http://blogs.love2d.org/content/let-it-glow-dynamically-adding-outlines-characters

outline_shader = [[
  vec4 resultCol;
  extern vec2 stepSize;

  vec4 effect( vec4 col, Image texture, vec2 texturePos, vec2 screenPos )
  {
    // get color of pixels:
    number alpha = 4*texture2D( texture, texturePos ).a;
    alpha -= texture2D( texture, texturePos + vec2( stepSize.x, 0.0f ) ).a;
    alpha -= texture2D( texture, texturePos + vec2( -stepSize.x, 0.0f ) ).a;
    alpha -= texture2D( texture, texturePos + vec2( 0.0f, stepSize.y ) ).a;
    alpha -= texture2D( texture, texturePos + vec2( 0.0f, -stepSize.y ) ).a;

    // calculate resulting color
    resultCol = vec4( 0.4f, 1.0f, 0.1f, alpha );
    // return color for current pixel
    return resultCol;
  }
]]