-- Window skins

gui_orb = love.graphics.newImage("gfx/system/orb.png")
gui_arrow = love.graphics.newImage("gfx/system/arrow.png")

gui_skins = { }

for _, skin in ipairs(love.filesystem.getDirectoryItems("gfx/system/window")) do
  gui_skins[string.sub(skin, 1, -5)] = love.graphics.newImage("gfx/system/window/" .. skin)
end

local gsw = gui_skins.basic:getWidth()
local gsh = gui_skins.basic:getHeight()
gui_skin_quad_w = gsw / 3
gui_skin_quad_h = gsh / 3

gui_skin_quad = {
  top = {
    left = love.graphics.newQuad(0, 0, gui_skin_quad_w, gui_skin_quad_h, gsw, gsh),
    center = love.graphics.newQuad(gui_skin_quad_w, 0, gui_skin_quad_w, gui_skin_quad_h, gsw, gsh),
    right = love.graphics.newQuad(gui_skin_quad_w * 2, 0, gui_skin_quad_w, gui_skin_quad_h, gsw, gsh)
  },
  center = {
    left = love.graphics.newQuad(0, gui_skin_quad_h, gui_skin_quad_w, gui_skin_quad_h, gsw, gsh),
    center = love.graphics.newQuad(gui_skin_quad_w, gui_skin_quad_h, gui_skin_quad_w, gui_skin_quad_h, gsw, gsh),
    right = love.graphics.newQuad(gui_skin_quad_w * 2, gui_skin_quad_h, gui_skin_quad_w, gui_skin_quad_h, gsw, gsh)
  },
  bottom = {
    left = love.graphics.newQuad(0, gui_skin_quad_h * 2, gui_skin_quad_w, gui_skin_quad_h, gsw, gsh),
    center = love.graphics.newQuad(gui_skin_quad_w, gui_skin_quad_h * 2, gui_skin_quad_w, gui_skin_quad_h, gsw, gsh),
    right = love.graphics.newQuad(gui_skin_quad_w * 2, gui_skin_quad_h * 2, gui_skin_quad_w, gui_skin_quad_h, gsw, gsh)
  }
}

-- GUI

gui = { }
gui.__index = gui

setmetatable(gui, {
    __call = function(base, ...)
      local self = setmetatable({ }, base)
      self:new(...)
      return self
    end
  })

function gui:new()
  self.next_id = 1
  self.elements = { }
  
  return self
end

function gui:update(dt)
  local del = { }
  
  for i, e in ipairs(self.elements) do
    e:update(dt)
    
    if e.onUpdate then
      e:onUpdate(dt)
    end
    
    if e.delete then
      table.insert(del, i)
    end
  end
  
  while #del > 0 do
    table.remove(self.elements, del[#del])
    table.remove(del)
  end
end

function gui:draw()
  for i, e in ipairs(self.elements) do
    e:draw()
    
    if e.onDraw then
      e:onDraw()
    end
  end
end

function gui:newID()
  self.next_id = self.next_id + 1
  
  return self.next_id - 1
end

function gui:getTopID()
  return self.elements[#self.elements].id
end

function gui:moveToTop(eid)
  for i, e in ipairs(self.elements) do
    if e.id == eid then
      table.remove(self.elements, i)
      table.insert(self.elements, e)
      break
    end
  end
end

function gui:moveToBottom(eid)
  for i, e in ipairs(self.elements) do
    if e.id == eid then
      table.remove(self.elements, i)
      table.insert(self.elements, 1, e)
      break
    end
  end
end

function gui:newElement(opt)
  table.insert(self.elements, gui_element(opt))
  
  return self.elements[#self.elements]
end

function gui:newWindow(opt)
  table.insert(self.elements, window(opt))
  
  return self.elements[#self.elements]
end

function gui:newDialWindow(opt)
  table.insert(self.elements, dial_window(opt))
  
  return self.elements[#self.elements]
end

function gui:newMenu(opt)
  table.insert(self.elements, menu(opt))
  
  return self.elements[#self.elements]
end

function gui:newMessage(opt)
  table.insert(self.elements, message(opt))
  
  return self.elements[#self.elements]
end