--[[  self.test_win.arrow = {
    right = true,
    left = true
  }
  self.test_win.onUpdate = function(self)
    if self.top then
      if control:wasPressed("right") then
        scene.tdial:add(1, 0, 100)
      elseif control:wasPressed("left") then
        scene.tdial:add(-1, 0, 100)
      end
    end
  end]]

-- GUI - Element -> Window -> Dial Window

dial_window = { }
dial_window.__index = dial_window

setmetatable(dial_window, {
    __index = window,
    __call = function(base, ...)
      local self = setmetatable({ }, base)
      self:new(...)
      return self
    end
  })

function dial_window:new(opt)
  self.dial = dial(opt.dial)
  
  opt.w = self.dial:getWidth() + 24
  opt.h = 60
  
  self.manual = opt.manual
  self.max_place = #self.dial.spinners
  self.place = 1
  self.skip_mod = (opt.dial.mod_spinner ~= nil)
  
  if not self.manual then
    opt.arrow = {
      up = true,
      down = true
    }
  end
  
  window.new(self, opt)
  
  return self
end

function dial_window:update(dt)
  window.update(self, dt)
  
  self.dial:update(dt)
  
  if self.top and self.state == "IDLE" then
    if self.manual then
      if control:wasPressed("left") then
        audio_sfx:play("moveCursor")
        self.place = math.min(self.max_place - ((self.skip_mod and 1) or 0), self.place + 1)
      elseif control:wasPressed("right") then
        audio_sfx:play("moveCursor")
        self.place = math.max(1, self.place - 1)
      elseif control:wasPressed("up") then
        audio_sfx:play("moveCursor")
        self.dial:mod(10 ^ (self.place - 1))
      elseif control:wasPressed("down") then
        audio_sfx:play("moveCursor")
        self.dial:mod(-10 ^ (self.place - 1))
      end
    else
      if self.dial.max then
        self.arrow.up = self.dial.value < self.dial.max
      end
      if self.dial.min then
        self.arrow.down = self.dial.value > self.dial.min
      end
      
      if control:wasPressed("up") then
        audio_sfx:play("moveCursor")
        self.dial:mod(1)
      elseif control:wasPressed("down") then
        audio_sfx:play("moveCursor")
        self.dial:mod(-1)
      elseif control:wasPressed('r') then
        audio_sfx:play("moveCursor")
        self.dial:mod(10)
      elseif control:wasPressed('l') then
        audio_sfx:play("moveCursor")
        self.dial:mod(-10)
      end
    end
    
    if control:wasPressed('b') and self.onB then
      audio_sfx:play("cancel")
      self:onB()
    end
  end
end

function dial_window:draw()
  window.draw(self)
  
  if self.top and self.state == "IDLE" and self.manual then
    if cursor_blink < .8 then
      drawIconCentered(17, self.x + ((self.max_place - (self.place - 1)) * 18), self.y + 2, math.pi / 2)
    else
      drawIconCentered(17, self.x + ((self.max_place - (self.place - 1)) * 18), self.y + 6, math.pi / 2)
    end
  end
end

function dial_window:canvasDraw()
  self.dial:draw()
end