-- GUI - Element -> Window -> Menu

menu = { }
menu.__index = menu

setmetatable(menu, {
    __index = window,
    __call = function(base, ...)
      local self = setmetatable({ }, base)
      self:new(...)
      return self
    end
  })

function menu:new(opt)
  if not opt then
    opt = { }
  end
  
  self.static = opt.static
  self.no_auto_size = opt.no_auto_size
  
  self.lr_jump = opt.lr_jump or 8
  self.no_rollover = opt.no_rollover
  self.freeform = opt.freeform
  self.freeform_lr = opt.freeform_lr
  
  self.no_selection_sound = opt.no_selection_sound
  self.no_cancel_sound = opt.no_cancel_sound
  
  self.option = 1
  self.options = { }
  self.next_y = 0
  self.min = {
    x = 0,
    y = 0
  }
  self.max = {
    x = 0,
    y = 0
  }
  self.canvas_tx = 0
  self.canvas_ty = 0
  
  self.customDraw = opt.customDraw
  
  self.changed = true
  
  window.new(self, opt)
  
  if opt.options then
    for i, o in ipairs(opt.options) do
      self:newOption(o)
    end
  end
  
  if opt.auto_h then
    self.max_h = opt.max_h
    
    self:resizeHeight()
  end
  
  if not self.arrow then
    self.arrow = {
      up = function()
        return self.canvas_ty > 0
      end,
      down = function()
        return self.canvas_ty < self.max.y - self.canvas:getHeight()
      end
    }
  end
  
  self.disabled = opt.disabled
  self.greyed_out = opt.greyed_out
  
  self.onHover = opt.onHover
  self.onA = opt.onA
  self.onB = opt.onB
  self.onX = opt.onX
  self.onY = opt.onY
  
  return self
end

function menu:update(dt)
  window.update(self, dt)
  
  for i, o in ipairs(self.options) do
    if (self.disabled and self:disabled(o)) or (o.disabled ~= nil and assess(o.disabled)) or (self.greyed_out and self:greyed_out(o)) or (o.greyed_out ~= nil and assess(o.greyed_out)) then
      o.alpha = .48
    else
      o.alpha = nil
    end
    
    if self.state ~= "HIDDEN" and o.y > self.canvas_ty - 80 and o.y < self.canvas:getHeight() + 80 + self.canvas_ty then
      o:update(dt)
    end
  end
  
  local cur_option = self.options[self.option]
  local changed = false
  local skipped = false
  
  if self.top and self.state == "IDLE" then
    if (not self.freeform) and cur_option then
      if control:wasPressed("down") then
        repeat
          self.option = self.option + 1
          
          if self.option > #self.options then
            if (not self.no_rollover) then
              self.option = 1
              changed = true
            else
              if skipped then
                self.option = #self.options - 1
              else
                self.option = #self.options
              end
            end
          else
            changed = true
          end
          
          if self.options[self.option].skip then
            skipped = true
          else
            skipped = false
          end
        until not skipped
        
        audio_sfx:play("moveCursor")
      elseif control:wasPressed("up") then
        repeat
          self.option = self.option - 1
          
          if self.option < 1 then
            if (not self.no_rollover) then
              self.option = #self.options
              changed = true
            else
              if skipped then
                self.option = 2
              else
                self.option = 1
              end
            end
          else
            changed = true
          end
          
          if self.options[self.option].skip then
            skipped = true
          else
            skipped = false
          end
        until not skipped
        
        audio_sfx:play("moveCursor")
      elseif control:wasPressed('l') then
        if #self.options >= self.lr_jump then
          if self.option == 1 and (not self.no_rollover) then
            self.option = #self.options
            self.changed = true
          else
            self.option = math.max(1, self.option - self.lr_jump)
            self.changed = true
          end
          
          while self:getCurrentOption().skip do
            self.option = self.option - 1
          end
        end
        
        audio_sfx:play("moveCursor")
      elseif control:wasPressed('r') then
        if #self.options >= self.lr_jump then
          if self.option == #self.options and (not self.no_rollover) then
            self.option = 1
            self.changed = true
          else
            self.option = math.min(#self.options, self.option + self.lr_jump)
            self.changed = true
          end
          
          while self:getCurrentOption().skip do
            self.option = self.option + 1
          end
        end
        
        audio_sfx:play("moveCursor")
      end
    elseif cur_option then
      if control:directionWasPressed() then
        audio_sfx:play("moveCursor")
        
        self.option = self:getNearestOption(control:directionWasPressed())
      elseif self.freeform_lr then
        if control:wasPressed('l') then
          audio_sfx:play("moveCursor")
          
          if #self.options >= self.lr_jump then
            self.option = math.max(1, self.option - self.lr_jump)
            self.changed = true
          end
        elseif control:wasPressed('r') then
          audio_sfx:play("moveCursor")
          
          if #self.options >= self.lr_jump then
            self.option = math.min(#self.options, self.option + self.lr_jump)
            self.changed = true
          end
        end
      end
    end
    
    if control:wasPressed('a') then
      if cur_option and assess(cur_option.disabled) then
        audio_sfx:play("disabled")
      else
        if not self.no_selection_sound then
          audio_sfx:play("ring")
        end
        
        if self.onA and (not cur_option.override_a) then
          self:onA(cur_option)
        end
        
        if cur_option.onA then
          cur_option:onA(self)
        end
      end
    elseif control:wasPressed('b') then
      if not self.no_cancel_sound then
        audio_sfx:play("cancel")
      end
      
      if self.onB and (cur_option == nil or (not cur_option.override_b)) then
        self:onB(cur_option)
      end
      
      if cur_option and cur_option.onB then
        cur_option:onB(self)
      end
    elseif control:wasPressed('x') then
      if self.onX and (not cur_option.override_x) then
        self:onX(cur_option)
      end
      
      if cur_option.onX then
        cur_option:onX(self)
      end
    elseif control:wasPressed('y') then
      if self.onY and (not cur_option.override_y) then
        self:onY(cur_option)
      end
      
      if cur_option.onY then
        cur_option:onY(self)
      end
    end
  end
  
  if self.changed then
    if self.onHover and (cur_option == nil or (not cur_option.override_hover)) then
      self:onHover(cur_option)
    end
    
    if cur_option and cur_option.onHover then
      cur_option:onHover(self)
    end
    
    self.changed = false
  end
  
  cur_option = self.options[self.option]
  if (self.canvas_ty_reset_option and self.option == self.canvas_ty_reset_option) or self.option == 1 then
    self.canvas_ty = 0
  elseif cur_option.y + cur_option:getHeight() > self.canvas_ty + self.canvas:getHeight() then
    self.canvas_ty = cur_option.y + cur_option:getHeight() - self.canvas:getHeight()
  elseif cur_option.y < self.canvas_ty then
    self.canvas_ty = cur_option.y
  end
  
  if changed then
    self.changed = true
  end
end

function menu:draw()
  window.draw(self)
  
  local cur_option = self.options[self.option]
  if self.top and self.state == "IDLE" and cur_option then
    if cursor_blink < .8 then
      drawIcon(17, self.x - 14 + cur_option.x, self.y + (cur_option:getHeight() / 2) + cur_option.y - self.canvas_ty)
    else
      drawIcon(17, self.x - 18 + cur_option.x, self.y + (cur_option:getHeight() / 2) + cur_option.y - self.canvas_ty)
    end
  end
end

function menu:canvasDraw()
  love.graphics.push()
    love.graphics.translate(-self.canvas_tx, -self.canvas_ty)
    
    if self.customDraw then
      self:customDraw()
    end
    
    for i, o in ipairs(self.options) do
      if self.state ~= "HIDDEN" and o.y > self.canvas_ty - 80 and o.y < self.canvas:getHeight() + 80 + self.canvas_ty then
        o:draw()
      end
    end
  love.graphics.pop()
end

function menu:moveToTop()
  window.moveToTop(self)
  
  self.changed = true
  
  return self
end

function menu:resizeHeight()
  self.th = math.min(self.max_h or love.graphics.getHeight(), self.max.y + 24)
  self.y = self.ty + (self.th / 2)
  
  self.canvas = love.graphics.newCanvas(self.tw - 24, self.th - 24)
  
  return self
end

function menu:getNearestOption(dir)
  local cur_option = self.options[self.option]
  
  local good = {0, 9999, 9999}
  
  for i, o in ipairs(self.options) do
    if i ~= self.option then
      local dist, dist_x, dist_y = math.dist(cur_option:getCenterX(), cur_option:getCenterY(), o:getCenterX(), o:getCenterY())
      
      if (dir == "down" and dist_y > 0) or (dir == "up" and dist_y < 0) then
        if o:getCenterX() >= cur_option:getCenterX() - canvasWidth(.1) and o:getCenterX() <= cur_option:getCenterX() + canvasWidth(.1) then
          if good[1] == 0 or dist < good[2] then
            good = {i, dist}
          end
        end
      elseif (dir == "right" and dist_x > 0) or (dir == "left" and dist_x < 0) then
        if o:getCenterY() >= cur_option:getCenterY() - canvasWidth(.1) and o:getCenterY() <= cur_option:getCenterY() + canvasWidth(.1) then
          if good[1] == 0 or dist < good[2] then
            good = {i, dist}
          end
        end
      end
    end
  end
  
  return ((good[1] ~= 0 and good[1]) or self.option)
end

function menu:getCurrentOption()
  if self.options[self.option] then
    return self.options[self.option]
  end
end

function menu:clear(start)
  self.option = start or 1
  self.options = { }
  self.next_y = 0
  self.max = {
    x = 0,
    y = 0
  }
  self.canvas_ty_reset_option = start
  
  return self
end

function menu:newOption(opt)
  opt.x = (opt.x or 0) + 8
  opt.y = (opt.y or self.next_y)
  opt.w = self.tw - 32
  opt.instant = true
  
  local new_text = text(opt)
  
  new_text.disabled = opt.disabled
  new_text.greyed_out = opt.greyed_out
  new_text.skip = opt.skip
  new_text.onA = opt.onA
  new_text.override_a = opt.override_a
  
  new_text.onHover = opt.onHover
  
  new_text.data = opt.data
  
  table.insert(self.options, new_text)
  
  self.next_y = new_text.y + new_text:getHeight()
  
  if new_text.x < self.min.x then
    self.min.x = new_text.x
  end
  if new_text.x + new_text:getWidth() > self.max.x then
    self.max.x = new_text.x + new_text:getWidth()
  end
  if new_text.y < self.min.y then
    self.min.y = new_text.y
  end
  if new_text.y + new_text:getHeight() > self.max.y then
    self.max.y = new_text.y + new_text:getHeight()
  end
  
  return self
end