-- GUI - Element -> Window -> Message

message = { }
message.__index = message

setmetatable(message, {
    __index = window,
    __call = function(base, ...)
      local self = setmetatable({ }, base)
      self:new(...)
      return self
    end
  })

function message:new(opt)
  if not opt then
    opt = { }
  end
  
  self.static = opt.static
  self.no_auto_size = opt.no_auto_size
  
  self.text = 1
  self.text_queue = { }
  
  if opt.queue then
    local max_w = 0
    local max_h = 0
    
    for i, t in ipairs(opt.queue) do
      t.w = opt.w - 24
      local new_text = text(t)
      table.insert(self.text_queue, new_text)
      
      if new_text:getWidth() + 24 > max_w then
        max_w = new_text:getWidth() + 24
      end
      if new_text:getHeight() + 24 > max_h then
        max_h = new_text:getHeight() + 24
      end
    end
    
    if not self.no_auto_size then
      opt.w = max_w
      opt.h = max_h
    end
  else
    opt.text.w = opt.w - 24
    local new_text = text(opt.text)
    table.insert(self.text_queue, new_text)
    
    if not self.no_auto_size then
      opt.w = new_text:getWidth() + 24
      opt.h = new_text:getHeight() + 24
    end
  end
  
  window.new(self, opt)
  
  --self.state = "OPEN"
  
  return self
end

function message:update(dt)
  window.update(self, dt)
  
  local cur_text = self.text_queue[self.text]
  cur_text:update(dt)
  
  if self.state == "IDLE" then
    if not self.static then
      if cur_text:isHolding() then
        if control:wasPressed('a') then
          cur_text.hold = 0
        end
      end
      
      if cur_text:isFinished() then
        if self.text == #self.text_queue then
          self.state = "CLOSE"
        else
          self.text = self.text + 1
        end
      end
    end
  end
end

function message:draw()
  window.draw(self)
  
  local orb_x = self.x + self.w - 24
  local orb_y = self.y + self.h - 15
  
  local cur_text = self.text_queue[self.text]
  
  if self.off_canvas and self.skin == "none" then
    orb_x = self.x + cur_text:getWidth() + 12
    orb_y = self.y + cur_text:getHeight()
  end
  
  if self.state == "IDLE" and cur_text:isHolding() then
    if cursor_blink < .8 then
      love.graphics.setColor(HSLtoRGB(settings.window.h, settings.window.s, settings.window.l))
      love.graphics.draw(gui_orb, orb_x, orb_y)
    else
      love.graphics.setColor(HSLtoRGB(settings.window.h + .75, settings.window.s, settings.window.l))
      love.graphics.draw(gui_orb, orb_x, orb_y + 3)
    end
  end
end

function message:canvasDraw()
  local cur_text = self.text_queue[self.text]
  
  cur_text:draw()
end