--[[
  ProjectBapho.net
  
  Author: Hikitsune-Red =^~^=
]]--

-- System variables

start_time = love.timer.getTime()
version = "0.03    (11.4.0)"
love_version = { }
love_version.major, love_version.minor, love_version.revision, love_version.codename = love.getVersion()

local gfx_info = { }
gfx_info.name, gfx_info.version, gfx_info.vendor, gfx_info.device = love.graphics.getRendererInfo()

-- Require necessary libraries and modules

require("snippets")
require("trace")

log("Loading libraries:")

log("  ╠ UTF8")
utf8 = require("utf8")
log("  ╠ Socket")
socket = require("socket")
--log("  ╠ HTTPS")
--https = require("https")
log("  ╠ UUID")
uuid = require("lib.uuid")
uuid.seed()
log("  ╠ Serpent")
serpent = require("lib.serpent.src.serpent")
log("  ╠ QR Encode")
qrencode = require("lib.qrencode")
log("  ╠ Anim8")
anim8 = require("lib.anim8.anim8")

love.graphics.setDefaultFilter("nearest", "nearest")

local require_folders = {"core", "engine"}
for i, f in ipairs(require_folders) do
  local list = love.filesystem.getDirectoryItems(f)
  for j, l in ipairs(list) do
    require(f .. "." .. string.sub(l, 1, -5))
  end
end

log("  ╚ Complete!")

--[[for i, f in ipairs(love.filesystem.getDirectoryItems("obfuscated")) do
  local data = love.filesystem.read("obfuscated/" .. f)
  pcall(load(unobfuscate(data, true)))
end]]

log("Loading fonts...")
fonts = {
  fira_code = love.graphics.newFont("fonts/Fira_Code/static/FiraCode-SemiBold.ttf", 20),
  
  kubasta = love.graphics.newFont("fonts/kubasta/Kubasta.otf", 20),
  
  matchup = love.graphics.newFont("fonts/matchup-pro-v1.1/MatchupPro.otf", 32),
}

system_font = fonts.fira_code
game_font = fonts.matchup
log("Fonts loaded!")

log("Indexing cursors...")
cursors = {
  arrow = love.mouse.getSystemCursor("arrow"),
  hand = love.mouse.getSystemCursor("hand")
}
log("Cursors indexed!")

initIcons()

game_scale = 3
--game_canvas = love.graphics.newCanvas(384 * game_scale, 216 * game_scale)
game_canvas = love.graphics.newCanvas(1280, 720)
game_canvas:setFilter("linear", "linear")

-- Begin runtime process

_init_tick = true
start_time = love.timer.getTime()
roll_seed = true
seed_rolled = 0

local old_uuid = uuid
function uuid()
  if roll_seed then
    log("UUID Gen. Seed Locked after " .. seed_rolled .. " rolls: " .. uuid_gen_seed)
    roll_seed = false
  end
  
  return old_uuid(uuid_gen_seed)
end

function love.load(args)
  log(" ")
  log("|================================================|")
  log("|  Project Sirena")
  log("|  © Hikitsune-Red - 2024")
  log("|  Game Version " .. version)
  log("|  LÖVE Version " .. love_version.major .. "." .. love_version.minor .. "." .. love_version.revision .. " - \"" .. love_version.codename .. "\"")
  log("|================================================|")
  log("|  OS:")
  log("|      " .. love.system.getOS())
  log("|  Processor Count:")
  log("|      " .. love.system.getProcessorCount())
  log("|  Graphics:")
  log("|      " .. gfx_info.name)
  log("|      " .. gfx_info.device)
  log("|      " .. gfx_info.version)
  log("|================================================|")
  log(" ")
  
  log("LÖVE loaded! Game start!")
  
  ac = { }
  
  system = {
    glitching = false,
    
    -- Terminal stuff
    
    history = { },
    history_index = 0
  }
  
  settings = {
    lang = "en",
    
    volume = 1,
    volume_bgm = .8,
    volume_bgs = 1,
    volume_sfx = 1,
    
    draw_mode = "fit",
    
    window = {
      h = .56,
      s = 1,
      l = .64
    },
    
    top_down_trace = true,
    
    verbose = false,
    show_object_data = false,
  }
  
  cursor_blink = 0
  
  --[[outline_shader_proc = love.graphics.newShader(outline_shader)
  glitch_shader_proc = love.graphics.newShader(glitch_shader)
  glitch_shader_proc:send("strength_1", 0.08)
  glitch_shader_proc:send("glitch", true)]]
  
  log(" ")
  audio_bgm:init()
  log(" ")
  audio_sfx:init()
  log(" ")
  audio_stinger:init()
  log(" ")
  
  scene_manager:newScene("sound_test")
  audio_bgm:randomize(true)
end

-- LÖVE callbacks

function love.update(dt)
  if roll_seed then
    uuid_gen_seed = ""
    for i = 1, 12 do
      uuid_gen_seed = uuid_gen_seed .. randomHexChar()
      if i ~= 12 and i % 2 == 0 then
        uuid_gen_seed = uuid_gen_seed .. ":"
      end
    end
    seed_rolled = seed_rolled + 1
  end
  
  scene_manager:update(dt)
  
  --[[if control:keyPressed('g') then
    if love.keyboard.isDown("lshift") or love.keyboard.isDown("rshift") then
      system.glitching = not system.glitching
    end
  end]]
  
  --[[if control:keyPressed('o') then
    if roll_seed then
      print("UUID Gen. Seed Locked after " .. seed_rolled .. " rolls: " .. uuid_gen_seed)
      roll_seed = false
    end
    print(uuid(uuid_gen_seed))
  end]]
  
  if control:keyPressed('`') then
    scene_manager:newScene("terminal")
  end
  
  if control:keyPressed("f2") then
    settings.show_object_data = not settings.show_object_data
  end
  
  if control:keyPressed("f6") then
    if love.keyboard.isDown("lshift") then
      scene_manager:endScene()
    else
      scene_manager:reloadScene()
    end
  end
  
  if control:keyPressed("f11") then
    love.window.setFullscreen(not love.window.getFullscreen())
  end
  
  cursor_blink = (cursor_blink + dt) % 1.6
  --glitch_shader_proc:send("millis", love.timer.getTime() - start_time)
  --glitch_shader_proc:send("glitch", true)
  scene_manager.just_ended_scene = false
  updateTimers(dt)
  updateTrace(dt)
  audio_bgm:update(dt)
  audio_sfx:update(dt)
  audio_stinger:update(dt)
  control:update(dt)
end

function love.draw()
  love.graphics.setCanvas(game_canvas)
    love.graphics.clear(color("near_black"))
    scene_manager:draw()
  love.graphics.setCanvas()
  
  setColor("white")
  if system.glitching then
    --love.graphics.setShader(glitch_shader_proc)
  end
  if settings.draw_mode == "original_shrunk" then
    love.graphics.draw(game_canvas, scale.screenWidth(.5) - (scale.gameWidth(.5) / game_scale), scale.screenHeight(.5) - (scale.gameHeight(.5) / game_scale), 0, 1 / game_scale)
  elseif settings.draw_mode == "original" then
    love.graphics.draw(game_canvas, scale.screenWidth(.5) - scale.gameWidth(.5), scale.screenHeight(.5) - scale.gameHeight(.5))
  elseif settings.draw_mode == "stretch" then
    love.graphics.draw(game_canvas, 0, 0, 0, ratio.gameToScreen())
  elseif settings.draw_mode == "fit" then
    --local w, h = love.graphics.getWidth() - game_canvas:getWidth(), love.graphics.getHeight() - game_canvas:getHeight()
    local w, h = ratio.gameToScreen()
    
    if w > h then
      love.graphics.draw(game_canvas, scale.screenWidth(.5) - (scale.gameWidth(.5) * h), 0, 0, h)
    elseif h > w then
      love.graphics.draw(game_canvas, 0, scale.screenHeight(.5) - (scale.gameHeight(.5) * w), 0, w)
    else
      love.graphics.draw(game_canvas, 0, 0, 0, ratio.gameToScreen())
    end
  elseif settings.draw_mode == "maintain" then
    local w, h = ratio.gameToScreen()
    if w == 1 and h == 1 then
      love.graphics.draw(game_canvas)
    else
      local aspect_ratio = { }
      aspect_ratio.w, aspect_ratio.h = scale.game(1 / game_scale)
      
      local width_scale, height_scale = ratio.screenWidth(aspect_ratio.w), ratio.screenHeight(aspect_ratio.h)
      local aspect_scale = math.floor(math.min(width_scale, height_scale)) / game_scale
      
      love.graphics.draw(game_canvas, (scale.screenWidth(1) - scale.gameWidth(aspect_scale)) / 2, (scale.screenHeight(1) - scale.gameHeight(aspect_scale)) / 2, 0, aspect_scale)
    end
  end
  --love.graphics.setShader()
  
  scene_manager:drawSystem()
  
  drawTrace()
  
  --love.graphics.print("FPS: " .. love.timer.getFPS() .. "\n" .. serpent.block(control.cues_down), 0, 0)
  --[[local mx, my = getMouseAdjusted()
  local rw, rh = ratio.gameToScreen()
  love.graphics.print("R: " .. rw .. ", " .. rh .. "\nMX: " .. mx .. ", MY: " .. my)]]
end

function getMouseAdjusted()
  local mxa, mya = 0, 0
  local mx, my = love.mouse.getPosition()
  
  --[[if settings.draw_mode == "original_shrunk" then
    love.graphics.draw(game_canvas, scale.screenWidth(.5) - (scale.gameWidth(.5) / game_scale), scale.screenHeight(.5) - (scale.gameHeight(.5) / game_scale), 0, 1 / game_scale)
  elseif settings.draw_mode == "original" then
    love.graphics.draw(game_canvas, scale.screenWidth(.5) - scale.gameWidth(.5), scale.screenHeight(.5) - scale.gameHeight(.5))
    
    mxa = mx 
  elseif settings.draw_mode == "stretch" then
    love.graphics.draw(game_canvas, 0, 0, 0, ratio.gameToScreen())
  elseif settings.draw_mode == "fit" then
    
  else]]if settings.draw_mode == "maintain" then
    local w, h = ratio.gameToScreen()
    if w == 1 and h == 1 then
      mxa = math.ceil(mx / w)
      mya = math.ceil(my / h)
    else
      local aspect_ratio = { }
      aspect_ratio.w, aspect_ratio.h = scale.game(1 / game_scale)
      
      local width_scale, height_scale = ratio.screenWidth(aspect_ratio.w), ratio.screenHeight(aspect_ratio.h)
      local aspect_scale = math.floor(math.min(width_scale, height_scale)) / game_scale
      
      if mx < (scale.screenWidth(1) - scale.gameWidth(aspect_scale)) / 2 then
        mxa = 0
      elseif mx > ((scale.screenWidth(1) - scale.gameWidth(aspect_scale)) / 2) + scale.gameWidth(aspect_scale) then
        mxa = game_canvas:getWidth()
      else
        mxa = math.round((mx - (scale.screenWidth(1) - scale.gameWidth(aspect_scale)) / 2) / aspect_scale)
      end
      if my < (scale.screenHeight(1) - scale.gameHeight(aspect_scale)) / 2 then
        mya = 0
      elseif my > ((scale.screenHeight(1) - scale.gameHeight(aspect_scale)) / 2) + scale.gameHeight(aspect_scale) then
        mya = game_canvas:getHeight()
      else
        mya = math.round((my - (scale.screenHeight(1) - scale.gameHeight(aspect_scale)) / 2) / aspect_scale)
      end
    end
  end
  
  return mxa, mya
end

function love.resize(w, h)
  --game_canvas = love.graphics.newCanvas(w, h)
end

local old_mr = math.random

function math.random(...)
  err("Call to \"math.random()\"!")
  
  return old_mr(...)
end