-- Options Menu

local scene = { }

function scene:load(opt)
  self.gui = gui()
  
  love.graphics.setFont(game_font)
  
  self.title_window2 = self.gui:newMessage({text = {text = "none", instant = true}, static = true, no_auto_size = true, x = canvasWidth(.2), w = canvasWidth(.2), h = 54})
  self.title_window = self.gui:newMessage({text = {text = {"/icon 74", "Options"}, instant = true}, static = true, no_auto_size = true, w = canvasWidth(.2), h = 54}):open()
  self.title_window.arrow = {
    right = function()
      return self.title_window2.state ~= "HIDDEN"
    end
  }
  
  self.description_window = self.gui:newWindow({x = canvasWidth(.7), y = self.title_window.th, w = canvasWidth(.3), h = canvasHeight() - self.title_window.th}):open()
  self.description_window.onUpdate = function(self, dt)
    if self.text then
      self.text:update(dt)
    end
  end
  self.description_window.canvasDraw = function(self)
    if self.text then
      self.text:draw()
    end
  end
  
  local desc_w = scene.description_window.tw - 24
  
  self.category_window = self.gui:newMenu({
      y = self.title_window.th,
      w = canvasWidth(.2),
      h = canvasHeight() - self.title_window.th,
      auto_h = true,
      options = {
        {text = {"/icon 434", "Audio"},
          data = text({text = "Settings that control the volume of sounds and music.", w = desc_w, instant = true}),
          onA = function()
            scene.options_window:clear()
            scene.options_window:newOption({
                text = "Master Volume",
                data = text({text = "Controls the overall volume of the game.", w = desc_w, instant = true}),
                onA = function(self, win)
                  local dial_win = scene.gui:newDialWindow({
                      dial = {
                        value = settings.volume * 100,
                        min = 0,
                        max = 100,
                        spinners = 3,
                        mod_spinner = "percent",
                        independent_spinners = true
                      },
                      x = win.tx + self.x + self:getWidth() + 16,
                      y = win.ty + self.y - 4,
                      onUpdate = function(self, dt)
                        settings.volume = self.dial.value / 100
                      end,
                      manual = true
                    })
                  dial_win.onB = function(self)
                    self:close(true)
                  end
                  
                  dial_win:open()
                end
              })
            scene.options_window:newOption({
                text = "BGM Volume",
                data = text({text = "Controls the volume of background and fanfare music.", w = desc_w, instant = true}),
                onA = function(self, win)
                  local dial_win = scene.gui:newDialWindow({
                      dial = {
                        value = settings.volume_bgm * 100,
                        min = 0,
                        max = 100,
                        spinners = 3,
                        mod_spinner = "percent",
                        independent_spinners = true
                      },
                      x = win.tx + self.x + self:getWidth() + 16,
                      y = win.ty + self.y - 4,
                      onUpdate = function(self, dt)
                        settings.volume_bgm = self.dial.value / 100
                      end,
                      manual = true
                    })
                  dial_win.onB = function(self)
                    self:close(true)
                  end
                  
                  dial_win:open()
                end
              })
            scene.options_window:newOption({
                text = "BGS Volume",
                data = text({text = "Controls the volume of background sound effects like rain.", w = desc_w, instant = true}),
                onA = function(self, win)
                  local dial_win = scene.gui:newDialWindow({
                      dial = {
                        value = settings.volume_bgs * 100,
                        min = 0,
                        max = 100,
                        spinners = 3,
                        mod_spinner = "percent",
                        independent_spinners = true
                      },
                      x = win.tx + self.x + self:getWidth() + 16,
                      y = win.ty + self.y - 4,
                      onUpdate = function(self, dt)
                        settings.volume_bgs = self.dial.value / 100
                      end,
                      manual = true
                    })
                  dial_win.onB = function(self)
                    self:close(true)
                  end
                  
                  dial_win:open()
                end
              })
            scene.options_window:newOption({
                text = "SFX Volume",
                data = text({text = "Controls the volume of sound effects.", w = desc_w, instant = true}),
                onA = function(self, win)
                  local dial_win = scene.gui:newDialWindow({
                      dial = {
                        value = settings.volume_sfx * 100,
                        min = 0,
                        max = 100,
                        spinners = 3,
                        mod_spinner = "percent",
                        independent_spinners = true
                      },
                      x = win.tx + self.x + self:getWidth() + 16,
                      y = win.ty + self.y - 4,
                      onUpdate = function(self, dt)
                        settings.volume_sfx = self.dial.value / 100
                      end,
                      manual = true
                    })
                  dial_win.onB = function(self)
                    self:close(true)
                  end
                  
                  dial_win:open()
                end
              })
            
            scene.options_window:resizeHeight()
          end},
        
        {text = {"/icon 455", "Video"},
          data = text({text = "Settings that control the screen resolution and scaling.", w = desc_w, instant = true}),
          onA = function()
            scene.options_window:clear()
            scene.options_window.option = 2
            scene.options_window.canvas_ty_reset_option = 2
            
            scene.options_window:newOption({
                text = {colorTable("teal"), "---- Screen ----"},
                skip = true
              })
            scene.options_window:newOption({
                text = "Fullscreen Mode",
                greyed_out = function()
                  return not love.window.getFullscreen()
                end,
                data = text({text = "Toggles fullscreen mode.", w = desc_w, instant = true}),
                onA = function(self, win)
                  love.window.setFullscreen(not love.window.getFullscreen())
                end
              })
            scene.options_window:newOption({
                text = "VSync",
                greyed_out = function()
                  return love.window.getVSync() == 0
                end,
                data = text({text = "Toggles vertical synchronization on or off.", w = desc_w, instant = true}),
                onA = function(self, win)
                  if love.window.getVSync() == 0 then
                    love.window.setVSync(1)
                  else
                    love.window.setVSync(0)
                  end
                end
              })
            scene.options_window:newOption({
                text = {colorTable("teal"), "---- Window ----"},
                skip = true
              })
            scene.options_window:newOption({
                text = "Color: Hue",
                data = text({text = "Controls the color hue of in-game windows.", w = desc_w, instant = true}),
                onA = function(self, win)
                  local dial_win = scene.gui:newDialWindow({
                      dial = {
                        value = settings.window.h * 100,
                        min = 0,
                        max = 100,
                        spinners = 3,
                        independent_spinners = true
                      },
                      x = win.tx + self.x + self:getWidth() + 16,
                      y = win.ty + self.y - 4 - win.canvas_ty,
                      onUpdate = function(self, dt)
                        settings.window.h = self.dial.value / 100
                      end,
                      manual = true
                    })
                  dial_win.onB = function(self)
                    self:close(true)
                  end
                  
                  dial_win:open()
                end
              })
            scene.options_window:newOption({
                text = "Color: Saturation",
                data = text({text = "Controls the color saturation of in-game windows.", w = desc_w, instant = true}),
                onA = function(self, win)
                  local dial_win = scene.gui:newDialWindow({
                      dial = {
                        value = settings.window.s * 100,
                        min = 0,
                        max = 100,
                        spinners = 3,
                        independent_spinners = true
                      },
                      x = win.tx + self.x + self:getWidth() + 16,
                      y = win.ty + self.y - 4 - win.canvas_ty,
                      onUpdate = function(self, dt)
                        settings.window.s = self.dial.value / 100
                      end,
                      manual = true
                    })
                  dial_win.onB = function(self)
                    self:close(true)
                  end
                  
                  dial_win:open()
                end
              })
            scene.options_window:newOption({
                text = "Color: Luminosity",
                data = text({text = "Controls the color luminosity of in-game windows.", w = desc_w, instant = true}),
                onA = function(self, win)
                  local dial_win = scene.gui:newDialWindow({
                      dial = {
                        value = settings.window.l * 100,
                        min = 0,
                        max = 100,
                        spinners = 3,
                        independent_spinners = true
                      },
                      x = win.tx + self.x + self:getWidth() + 16,
                      y = win.ty + self.y - 4 - win.canvas_ty,
                      onUpdate = function(self, dt)
                        settings.window.l = self.dial.value / 100
                      end,
                      manual = true
                    })
                  dial_win.onB = function(self)
                    self:close(true)
                  end
                  
                  dial_win:open()
                end
              })
            
            scene.options_window:resizeHeight()
          end},
        
        {text = {"/icon 121", "Controls"},
          disabled = true,
          data = text({text = "Change controller/key mapping.", w = desc_w, instant = true}),
          onA = function()
            scene.options_window:clear()
          end},
        
        {text = {"/icon 1073", "System"},
          data = text({text = "Settings that control a variety of system functions.", w = desc_w, instant = true}),
          onA = function()
            scene.options_window:clear(2)
            
            scene.options_window:newOption({
                text = {colorTable("teal"), "---- Debugging ----"},
                skip = true
              })
            scene.options_window:newOption({
                text = "Open Save Directory",
                data = text({text = {"/instant true", "Opens the game\'s save directory in your file browser.", "/instant false", "/nl", "/nl", "/hold 8", colorTable("dark grey"), "/glitch true", "/time .1", "(Now,", "/hold .16", " why would you want to do that,", "/hold .16", " I wonder?)", "/glitch false", "/time"}, w = desc_w}),
                --[[disabled = function()
                  return love.system.getOS() ~= "Windows"
                end,]]
                onA = function()
                  love.system.openURL("file://" .. love.filesystem.getSaveDirectory())
                end
              })
            scene.options_window:newOption({
                text = "Dump Log",
                data = text({text = {"Outputs the system log to the save directory. The file this creates is very useful, and you should pass it along to the developers when your game is not behaving properly.", "/nl", "/nl", colorTable("teal"), "NOTE:", colorTable("white"), " A log is auto-generated when the game crashes."}, w = desc_w, instant = true}),
                onA = function()
                  dumpLog()
                end
              })
            scene.options_window:newOption({
                text = "Top-down Tracing",
                data = text({text = "Toggles between top-down or bottom-up display modes for the system trace.", w = desc_w, instant = true}),
                greyed_out = function()
                  return not settings.top_down_trace
                end,
                onA = function()
                  settings.top_down_trace = not settings.top_down_trace
                end
              })
            scene.options_window:newOption({
                text = "Verbose Tracing",
                data = text({text = {"Enables nearly all system messages to show in the system trace, including things like ", colorTable("teal"), "warn()", colorTable("white"), " or ", colorTable("teal"), "log()", colorTable("white"), " calls which, by default, only show up in the log."}, w = desc_w, instant = true}),
                greyed_out = function()
                  return not settings.verbose
                end,
                onA = function()
                  settings.verbose = not settings.verbose
                end
              })
            
            scene.options_window:resizeHeight()
          end},
      },
      onHover = function(self, option)
        scene.descriptionHover(option, self)
      end,
      onA = function(self, option)
        scene.title_window2.text_queue = {text({text = option.text, instant = true})}
        scene.title_window2:forceOpen()
        scene.options_window:moveToTop()
        scene.options_window:forceOpen()
      end
    }):open()
  
  self.options_window = self.gui:newMenu({x = self.category_window.tw, y = self.category_window.ty, w = canvasWidth(.5), h = canvasHeight() - self.category_window.ty})
  self.options_window.onHover = function(self, option)
    scene.descriptionHover(option, self)
  end
  self.options_window.onB = function(self)
    scene.title_window2:forceClose()
    scene.category_window.changed = true
    self:forceClose()
  end
  
  self.category_window:moveToTop()
end

function scene:update(dt)
  self.gui:update(dt)
end

function scene:draw()
  setColor("white")
  
  self.gui:draw()
end

function scene.descriptionHover(option, window)
  if option then
    scene.description_window.text = option.data
  end
end

return scene