-- Sound Test scene

local scene = { }

function scene:load(opt)
  self.bitlogo = love.graphics.newImage("gfx/system/bitbybitlogo2022-horiz.png")
  
  self.qrcode = createQRCode("https://discord.com/invite/26xZ8e7nRG")
  
  self.gui = gui()
  
  self.title_window = self.gui:newMessage({
      text = {
        text = {"/icon 441", colorTable("blue"), "Bit by Bit Sound"},
        instant = true
      },
      static = true,
      no_auto_size = true,
      w = canvasWidth(.2),
      h = 54
    }):open()
  self.title_window.arrow = {
    right = function()
      return scene.category_window.state ~= "HIDDEN"
    end
  }
  
  self.category_window = self.gui:newMessage({
      text = {
        text = "none",
        instant = true
      },
      static = true,
      no_auto_size = true,
      x = canvasWidth(.2),
      w = canvasWidth(.2),
      h = 54
    })
  
  self.music_info_window = self.gui:newWindow({
      y = canvasHeight(.8),
      w = canvasWidth(.7),
      h = canvasHeight(.2)
    }):open()
  self.music_info_window.countdown_dial = dial({value = 0, spinners = 3, mod_spinner = "countdown", min = 0})
  self.music_info_window.time_dial = {
    minutes = dial({value = 0, spinners = 2, independent_spinners = true, min = 0}),
    seconds = dial({value = 0, spinners = 2, independent_spinners = true, min = 0, max = 60})
  }
  self.music_info_window.loopstart_dial = dial({value = 0, spinners = 3, mod_spinner = "countdown", min = 0})
  self.music_info_window.length_dial = {
    minutes = dial({value = 0, spinners = 2, min = 0}),
    seconds = dial({value = 0, spinners = 2, min = 0, max = 60})
  }
  --[[self.music_info_window.loopstart_length_dial = {
    minutes = dial({value = 0, spinners = 2, min = 0}),
    seconds = dial({value = 0, spinners = 2, min = 0, max = 60})
  }]]
  self.music_info_window.note_float = 0
  self.music_info_window.onUpdate = function(self, dt)
    self.note_float = (self.note_float + 8 * dt) % (2 * math.pi)
    
    self.countdown_dial:set(math.floor(audio_bgm.countdown))
    self.countdown_dial.neutral_zeroes = not audio_bgm.randomized
    self.countdown_dial:update(dt)
    
    local cur_time = audio_bgm:currentTime(true)
    self.time_dial.minutes:set(math.floor(cur_time / 60))
    self.time_dial.minutes:update(dt)
    self.time_dial.seconds:set(cur_time % 60)
    self.time_dial.seconds:update(dt)
    
    if audio_bgm.current_source_start and audio_bgm.current_source_start:isPlaying() then
      local ls_cur_len = audio_bgm:currentLoopstartLength()
      local ls_cur_time = ls_cur_len - audio_bgm:currentLoopstartTime()
      self.loopstart_dial.neutral_zeroes = false
      self.loopstart_dial:set(math.floor(ls_cur_time))
    else
      self.loopstart_dial.neutral_zeroes = true
      self.loopstart_dial:set(0)
    end
    self.loopstart_dial:update(dt)
    
    local cur_len = audio_bgm:currentLength(true)
    local cur_min = math.floor(cur_len / 60)
    local cur_sec = cur_len % 60
    
    if self.length_dial.minutes.value ~= cur_min then
      self.length_dial.minutes:set(cur_min)
    end
    self.length_dial.minutes:update(dt)
    if self.length_dial.seconds.value ~= cur_sec then
      self.length_dial.seconds:set(cur_sec)
    end
    self.length_dial.seconds:update(dt)
    
    --[[local ls_cur_min = math.floor(ls_cur_len / 60)
    local ls_cur_sec = ls_cur_len % 60
    if self.loopstart_length_dial.minutes.value ~= ls_cur_min then
      self.loopstart_length_dial.minutes:set(ls_cur_min)
    end
    self.loopstart_length_dial.minutes:update(dt)
    if self.loopstart_length_dial.seconds.value ~= ls_cur_sec then
      self.loopstart_length_dial.seconds:set(ls_cur_sec)
    end
    self.loopstart_length_dial.seconds:update(dt)]]
  end
  self.music_info_window.canvasDraw = function(self)
    self.time_dial.minutes:draw(94, 40)
    self.time_dial.seconds:draw(self.time_dial.minutes:getWidth() + 106, 40)
    
    self.length_dial.minutes:draw(94, 82)
    self.length_dial.seconds:draw(self.length_dial.minutes:getWidth() + 106, 82)
    
    self.loopstart_dial:draw(412, 40)
    
    self.countdown_dial:draw(412, 82)
    
    local song = audio_bgm:currentSong()
    
    love.graphics.setFont(game_font)
    if song.display_title ~= "Nothing" then
      drawIcon(448, 4 - 2 * math.cos(self.note_float), 2 + 2 * math.sin(self.note_float))
    else
      drawIcon(448, 4, 4)
    end
    love.graphics.printo({colorTable("lavender"), song.display_title, colorTable("white"), " by " .. song.artist}, 36, 6)
    love.graphics.printo("Time:     :", 24, 46)
    love.graphics.printo("Length:     :", 0, 88)
    love.graphics.printo("Loop begins in ", 224, 46)
    love.graphics.printo("Shuffling in ", 256, 88)
  end
  
  self.category_menu = self.gui:newMenu({
      y = scene.title_window.th,
      w = scene.title_window.tw,
      auto_h = true,
      options = {
        {text = {"/icon 444", "Music"},
          onA = function()
            scene.music_menu:moveToTop()
            scene.music_menu:move(scene.category_menu.tw)
            scene.music_meta_menu:move(nil, scene.title_window.th)
          end},
        {text = {colorTable("ice"), "----Want More?----"},
          skip = true},
        {text = {"/icon 1123", "Discord"},
          onA = function()
            love.system.openURL("https://discord.com/invite/26xZ8e7nRG")
          end},
        {text = {"/icon 1124", "Patreon"},
          onA = function()
            love.system.openURL("https://patreon.com/bitbybitsound")
          end},
        {text = {colorTable("ice"), "---- Player by ----"},
          skip = true},
        {text = {"/icon 1125", "Hikitsune-Red"},
          onA = function()
            love.system.openURL("https://hikitsune-red.itch.io")
          end}
      },
      onA = function(self)
        local cur_option = self.options[self.option]
        scene.category_window.text_queue = {text({text = cur_option.text, instant = true})}
        scene.category_window:moveToBottom():forceOpen()
      end
    }):open()
  
  --print(self.category_menu.th)
  --204
  
  self.about_window = self.gui:newMessage({
      text = {
        text = {colorTable("blue"), "      Controls", "/nl", colorTable("ice"), "Arrow Keys & Q / W", "/nl", colorTable("white"), " - Navigate & Jump", "/nl", colorTable("ice"), "Z / X", "/nl", colorTable("white"), " - Select / Cancel", "/nl", colorTable("ice"), "A", "/nl", colorTable("white"), " - Edit Tags", "/nl", colorTable("ice"), "C", "/nl", colorTable("white"), " - Jump to Current Song"},
        instant = true
      },
      y = scene.category_menu.ty + self.category_menu.th,
      w = scene.title_window.tw,
      h = canvasHeight() - (scene.category_menu.ty + self.category_menu.th + scene.music_info_window.th),
      static = true,
      no_auto_size = true
    }):open()
  
  local song_list = {
    {text = {"/rainbow true", "Random Song", "/rainbow false"},
      onA = function()
        audio_bgm:queue("random")
        audio_bgm:fadeOut()
      end},
    {text = "Shuffle",
      greyed_out = function()
        return not audio_bgm.randomized
      end,
      onA = function()
        audio_bgm:randomize()
      end},
    {text = {colorTable("red"), "Stop"},
      onA = function()
        audio_bgm:stop()
        audio_bgm:randomize(false)
      end},
    {text = "Clear Playlist",
      disabled = function()
        return not audio_bgm:hasPlaylist()
      end,
      onA = function()
        audio_bgm:clearPlaylist()
      end},
    {text = "------------------------------------------------",
      disabled = true,
      skip = true},
  }
  for i, filename in ipairs(audio_bgm.list) do
    --local song = audio_bgm.songs[filename]
    local no = {
      --text = song.display_title,
      text = filename,
      onA = function()
        audio_bgm:queue(filename)
        audio_bgm:fadeOut()
      end
    }
    
    table.insert(song_list, no)
  end
  
  self.music_menu = self.gui:newMenu({
      x = -canvasWidth(.7),
      y = scene.title_window.th,
      w = canvasWidth(.5),
      auto_h = true,
      max_h = canvasHeight() - scene.title_window.th - scene.music_info_window.th,
      options = song_list,
      onB = function(self)
        scene.music_meta_menu:move(nil, canvasHeight())
        scene.music_meta_menu:moveToBottom()
        self:move(-canvasWidth(.7), nil, 10)
        self:moveToBottom()
        scene.category_window:forceClose()
      end,
      onX = function(self)
        audio_sfx:play("ring")
        
        if self.option > 4 then
          scene.music_meta_menu.option = 12
          scene.music_meta_menu.changed = true
          scene.music_meta_menu:moveToTop()
        end
      end,
      onY = function(self)
        
      end
    }):open()
  
  self.music_meta_menu = self.gui:newMenu({
      x = canvasWidth(.7),
      y = canvasHeight(),
      w = canvasWidth(.3),
      h = canvasHeight() - scene.title_window.th,
      customDraw = function(self)
        local song = audio_bgm.songs[scene.music_menu:getCurrentOption().text]
        if song then
          love.graphics.printfo({colorTable("lavender"), song.display_title, colorTable("white"), "\nby " .. song.artist}, 2, 2, self.canvas:getWidth() - 4)
        end
        
        --love.graphics.printo({colorTable("teal"), "-Tags-"}, 0, 96)
      end,
      lr_jump = 0,
      options = {
        {text = {colorTable("teal"), "---- Era ----"},
          x = 0,
          y = 96,
          data = "ignore",
          skip = true},
        {text = "8bit",
          data = "8bit",
          skip = true},
        {text = "SNES / PSX",
          data = "snes",
          skip = true},
        {text = "Contemporary",
          data = "contemporary",
          skip = true},
        {text = {colorTable("teal"), "---- Type ----"},
          data = "ignore",
          skip = true},
        {text = "Battle",
          data = "battle",
          skip = true},
        --[[{text = "Boss Battle",
          data = "battle_boss"},]]
        {text = "Dungeon",
          data = "dungeon",
          skip = true},
        {text = "Event",
          data = "event",
          skip = true},
        {text = "Town",
          data = "town",
          skip = true},
        {text = "Travel",
          data = "travel",
          skip = true},
        --[[{text = "------------------------",
          skip = true},
        {text = "Ominous",
          data = "mood_ominous"},
        {text = "Somber",
          data = "mood_somber"},
        {text = "Upbeat",
          data = "mood_upbeat"},]]
        {text = {colorTable("teal"), "---- Tags ----"},
          data = "ignore",
          skip = true},
        {text = "Don\'t Fade In",
          data = "no_fade_in"},
        {text = "Don\'t Shuffle",
          data = "no_shuffle"},
        {text = "Short Shuffle",
          data = "short_shuffle"},
        {text = "Playlist",
          data = "ignore",
          greyed_out = function()
            local song = audio_bgm.songs[scene.music_menu:getCurrentOption().text]
            
            return (not audio_bgm:isOnPlaylist(song.title))
          end,
          override_a = true,
          onA = function(self, win)
            local song = audio_bgm.songs[scene.music_menu:getCurrentOption().text]
            
            if not audio_bgm:isOnPlaylist(song.title) then
              audio_bgm:addToPlaylist(song.title)
            else
              audio_bgm:removeFromPlaylist(song.title)
            end
          end},
        {text = {"/icon 113", colorTable("ice"), "If any songs are tagged \"Playlist\" they are the only songs being shuffled!"},
          data = "ignore",
          skip = true},
      },
      greyed_out = function(self, option)
        local song = audio_bgm.songs[scene.music_menu:getCurrentOption().text]
        
        if song then
          if option.data == "ignore" then
            return
          end
          
          return not song.tag[option.data]
        else
          return true
        end
      end,
      onA = function(self, option)
        local song = audio_bgm.songs[scene.music_menu:getCurrentOption().text]
        
        song.tag[option.data] = not song.tag[option.data]
      end,
      onB = function(self, option)
        scene.music_menu:moveToTop()
      end
    }):open()
  
  self.category_menu:moveToTop()
  --self.music_meta_menu:moveToTop()
end

function scene:update(dt)
  self.gui:update(dt)
  
  if control:keyPressed('c') then
    for i, s in ipairs(audio_bgm.list) do
      if s == audio_bgm:currentTitle(true) then
        self.music_menu.option = i + 5
        self.music_menu.changed = true
        break
      end
    end
  elseif control:keyPressed('r') then
    audio_bgm:queue("random")
    audio_bgm:fadeOut()
  end
end

function scene:draw()
  setColor("white", .5)
  local logo_scale = .25
  love.graphics.draw(self.bitlogo, canvasWidth(.6) - (self.bitlogo:getWidth() * logo_scale / 2), canvasHeight(.44) - (self.bitlogo:getHeight() * logo_scale / 2), 0, logo_scale)
  
  --[[love.graphics.printo("Join the Bit By Bit Sound\n      Discord Server!", canvasWidth(.735), canvasHeight(.8) - 60)
  love.graphics.draw(self.qrcode, canvasWidth(.8), canvasHeight(.8), 0, 4)]]
  
  setColor("white", 1)
  self.gui:draw()
end

return scene