local scene = {
  text_input_mode = true,
  no_suspension = true,
  system_canvas = true
}

function scene:load(opt)
  self.input_line = ""
  self.cursor_offset = -1
  self.cursor_char = "_"
  self.tick = 0
  self.y_offset = 0
end

function scene:update(dt)
  self.tick = (self.tick + dt) % .96
  
  if self.tick > .48 then
    self.cursor_char = " "
  else
    self.cursor_char = "_"
  end
end

function scene:draw()
  love.graphics.push()
    love.graphics.setFont(system_font)
    love.graphics.translate(0, -self.y_offset)
    local next_y = 0
    for i = math.max(1, #log_text - 41), #log_text do
      if i >= 1 then
        local l = log_text[i]
        love.graphics.printf(l, 2, next_y, ratio.screenWidth(1) - 4)
        local _, num_lines = fonts.fira_code:getWrap(l, ratio.screenWidth(1) - 4)
        num_lines = #num_lines
        next_y = next_y + (fonts.fira_code:getHeight() * num_lines)
      end
    end
    
    local input_text = "> " .. string.sub(self.input_line, 0, self.cursor_offset) .. self.cursor_char .. ((self.cursor_offset ~= -1 and string.sub(self.input_line, self.cursor_offset + 1, -1)) or "")
    love.graphics.printf(input_text, 2, next_y, ratio.screenWidth(1) - 4)
    local _, num_lines = fonts.fira_code:getWrap(input_text, ratio.screenWidth(1) - 4)
    num_lines = #num_lines
    
    if next_y + (system_font:getHeight() * num_lines) > ratio.screenHeight(1) then
      self.y_offset = next_y + (system_font:getHeight() * num_lines) - ratio.screenHeight(1)
    end
  love.graphics.pop()
end

function scene:textInput(text)
  self.input_line = string.sub(self.input_line, 0, self.cursor_offset) .. text .. ((self.cursor_offset ~= -1 and string.sub(self.input_line, self.cursor_offset + 1, -1)) or "")
end

function scene:keyPressed(key, scancode, is_repeat)
  if love.keyboard.isDown("lctrl") then
    if control:keyPressed('v') then
      self.input_line = self.input_line .. love.system.getClipboardText()
    end
  end
  
  if key == "backspace" and self.cursor_offset ~= -string.len(self.input_line) - 1 then
    -- get the byte offset to the last UTF-8 character in the string.
    --local byteoffset = utf8.offset(self.input_line, -1)
    local byteoffset = utf8.offset(self.input_line, self.cursor_offset)
    
    if byteoffset then
      -- remove the last UTF-8 character.
      -- string.sub operates on bytes rather than UTF-8 characters, so we couldn't do string.sub(text, 1, -2).
      self.input_line = string.sub(self.input_line, 1, byteoffset - 1) .. string.sub(self.input_line, byteoffset + 1, -1)
    end
  end
  
  if key == "left" then
    self.cursor_offset = math.max(self.cursor_offset - 1, -string.len(self.input_line) - 1)
  end
  
  if key == "right" then
    self.cursor_offset = math.min(self.cursor_offset + 1, -1)
  end
  
  if key == "home" then
    self.cursor_offset = -string.len(self.input_line) - 1
  end
  
  if key == "end" then
    self.cursor_offset = -1
  end
  
  if key == "up" and #system.history > 0 then
    system.history_index = math.min(#system.history, system.history_index + 1)
    self.input_line = system.history[#system.history - (system.history_index - 1)]
    self.cursor_offset = -1
  end
  
  if key == "down" then
    system.history_index = math.max(0, system.history_index - 1)
    if system.history_index == 0 then
      self.input_line = ""
    else
      self.input_line = system.history[#system.history - (system.history_index - 1)]
    end
    self.cursor_offset = -1
  end
  
  if key == "return" then
    log("> " .. self.input_line)
    local success, errmsg = pcall(load(self.input_line))
    if (not success) then
      err(errmsg)
    end
    table.insert(system.history, self.input_line)
    system.history_index = 0
    self.input_line = ""
    self.cursor_offset = -1
  end
  
  if key == "escape" then
    scene_manager:endScene()
  end
end

return scene