-- Shortcuts and snippets

-- Get global variable by name

function get(var)
  return _G[var]
end

-- Variable argument handling

function varg(...)
  local n = select("#", ...)
  local args = { }
  for i = 1, n do
    args[i] = (select(i, ...))
  end
  return args
end

function backspace(text)
  local byteoffset = utf8.offset(text, -1)
  
  if byteoffset then
    text = string.sub(text, 1, byteoffset - 1)
  end
  
  return text
end

--[[function utf8.sub(s, i, j)
  i = utf8.offset(s, i)
  j = utf8.offset(s, j + 1) -1
  return string.sub(s, i, j)
end]]

function random(min, max)
  if rng then
    return rng:random(min, max)
  else
    return love.math.random(min, max)
  end
end

function randomDirection()
  local ran = random(1, 4)
  
  if ran == 1 then
    return "up"
  elseif ran == 2 then
    return "right"
  elseif ran == 3 then
    return "left"
  elseif ran == 4 then
    return "down"
  end
end

function randomHexChar()
  local c = love.math.random(0, 15)
  
  if c > 9 then
    if c == 10 then c = "a" end
    if c == 11 then c = "b" end
    if c == 12 then c = "c" end
    if c == 13 then c = "d" end
    if c == 14 then c = "e" end
    if c == 15 then c = "f" end
  end
  
  return tostring(c)
end

function timeLengthString(time, as_text, no_seconds)
  local tis = time + 0
  local days = math.floor(tis / 86400)
  tis = tis - (days * 86400)
  local hours = math.floor(tis / 3600)
  tis = tis - (hours * 3600)
  local minutes = math.floor(tis / 60)
  tis = math.round(tis - (minutes * 60), 0)
  
  local ret = ""
  if as_text then
    if days > 0 then
      ret = ret .. days .. " Days " .. hours .. " Hours " .. minutes .. " Minutes "
    elseif hours > 0 then
      ret = ret .. hours .. " Hours " .. minutes .. " Minutes "
    elseif minutes > 0 then
      ret = ret .. minutes .. " Minutes "
    end
    
    if ret == "" or (not no_seconds) then
      ret = ret .. tis .. " Seconds"
    end
  else
    ret = ret .. string.format("%03d", days) .. ":" .. string.format("%02d", hours) .. ":" .. string.format("%02d", minutes)
    
    if not no_seconds then
      ret = ret .. ":" .. string.format("%02d", tis)
    end
  end
  
  return ret
end

function randomPosNeg()
  if love.math.random() >= .5 then
    return 1
  else
    return -1
  end
end

local dsdist = 2
local dscolor = "near_black"

function setDropShadowDist(dist)
  dsdist = dist or 1
end

function setDropShadowColor(color)
  dscolor = color
end

function love.graphics.printds(text, x, y, ...)
  local r, g, b, a = love.graphics.getColor()
  
  setColor(dscolor)
  love.graphics.print(text, x + dsdist, y + dsdist, ...)
  love.graphics.setColor(r, g, b, a)
  love.graphics.print(text, x, y, ...)
end

function love.graphics.printfds(text, x, y, ...)
  local r, g, b, a = love.graphics.getColor()
  
  setColor(dscolor)
  love.graphics.printf(text, x + dsdist, y + dsdist, ...)
  love.graphics.setColor(r, g, b, a)
  love.graphics.printf(text, x, y, ...)
end

local odist = 2

function setOutlineDist(dist)
  odist = dist
end

function getOutlineDist()
  return odist
end

function love.graphics.drawo(drawable, x, y, ...)
  local r, g, b, a = love.graphics.getColor()
  
  love.graphics.push()
    setColor(dscolor, a)
    love.graphics.draw(drawable, x + odist, y, ...)
    love.graphics.draw(drawable, x + odist, y + odist, ...)
    --love.graphics.draw(drawable, x + odist, y + odist + odist, ...)
    love.graphics.draw(drawable, x, y + odist, ...)
    love.graphics.draw(drawable, x, y + odist + odist, ...)
    love.graphics.draw(drawable, x - odist, y + odist, ...)
    --love.graphics.draw(drawable, x - odist, y + odist + odist, ...)
    love.graphics.draw(drawable, x - odist, y, ...)
    love.graphics.draw(drawable, x - odist, y - odist, ...)
    love.graphics.draw(drawable, x, y - odist, ...)
    love.graphics.draw(drawable, x + odist, y - odist, ...)
    love.graphics.setColor(r, g, b, a)
    love.graphics.draw(drawable, x, y, ...)
  love.graphics.pop()
end

function love.graphics.drawqo(drawable, quad, x, y, ...)
  local r, g, b, a = love.graphics.getColor()
  
  love.graphics.push()
    setColor(dscolor, a)
    love.graphics.draw(drawable, quad, x + odist, y, ...)
    love.graphics.draw(drawable, quad, x + odist, y + odist, ...)
    --love.graphics.draw(drawable, quad, x + odist, y + odist + odist, ...)
    love.graphics.draw(drawable, quad, x, y + odist, ...)
    love.graphics.draw(drawable, quad, x, y + odist + odist, ...)
    love.graphics.draw(drawable, quad, x - odist, y + odist, ...)
    --love.graphics.draw(drawable, quad, x - odist, y + odist + odist, ...)
    love.graphics.draw(drawable, quad, x - odist, y, ...)
    love.graphics.draw(drawable, quad, x - odist, y - odist, ...)
    love.graphics.draw(drawable, quad, x, y - odist, ...)
    love.graphics.draw(drawable, quad, x + odist, y - odist, ...)
    love.graphics.setColor(r, g, b, a)
    love.graphics.draw(drawable, quad, x, y, ...)
  love.graphics.pop()
end

function love.graphics.printo(text, x, y, ...)
  local r, g, b, a = love.graphics.getColor()
  
  love.graphics.push()
    love.graphics.translate(odist, -2)
    setColor(dscolor, a)
    love.graphics.print(text, x + odist, y, ...)
    love.graphics.print(text, x + odist, y + odist, ...)
    --love.graphics.print(text, x + odist, y + odist + odist, ...)
    love.graphics.print(text, x, y + odist, ...)
    love.graphics.print(text, x, y + odist + odist, ...)
    love.graphics.print(text, x - odist, y + odist, ...)
    --love.graphics.print(text, x - odist, y + odist + odist, ...)
    love.graphics.print(text, x - odist, y, ...)
    love.graphics.print(text, x - odist, y - odist, ...)
    love.graphics.print(text, x, y - odist, ...)
    love.graphics.print(text, x + odist, y - odist, ...)
    love.graphics.setColor(r, g, b, a)
    love.graphics.print(text, x, y, ...)
  love.graphics.pop()
end

function love.graphics.printfo(text, x, y, ...)
  local r, g, b, a = love.graphics.getColor()
  
  love.graphics.push()
    love.graphics.translate(odist, -2)
    setColor(dscolor, a)
    love.graphics.printf(text, x + odist, y, ...)
    love.graphics.printf(text, x + odist, y + odist, ...)
    --love.graphics.printf(text, x + odist, y + odist + odist, ...)
    love.graphics.printf(text, x, y + odist, ...)
    love.graphics.printf(text, x, y + odist + odist, ...)
    love.graphics.printf(text, x - odist, y + odist, ...)
    --love.graphics.printf(text, x - odist, y + odist + odist, ...)
    love.graphics.printf(text, x - odist, y, ...)
    love.graphics.printf(text, x - odist, y - odist, ...)
    love.graphics.printf(text, x, y - odist, ...)
    love.graphics.printf(text, x + odist, y - odist, ...)
    love.graphics.setColor(r, g, b, a)
    love.graphics.printf(text, x, y, ...)
  love.graphics.pop()
end

function love.graphics.drawCentered(drawable, x, y, r, sx, sy, ox, oy, ...)
  love.graphics.draw(drawable, x, y, r, sx, sy, (ox or 0) + drawable:getWidth() / 2, (oy or ox or 0) + drawable:getHeight() / 2, ...)
end

function math.round(num, decimal)
  local mult = 10 ^ (decimal or 0)
  return math.floor(num * mult + .5) / mult
end

function math.dist(x1, y1, x2, y2)
  return math.sqrt(((x2 - x1) ^ 2) + ((y2 - y1) ^ 2)), x2 - x1, y2 - y1
end

-- Get saveable data

function serialize(tab, pretty)
  if pretty or settings.pretty_saves then
    return serpent.block(tab, {name = "SaveData", comment = false, valtypeignore = {["function"] = true}})
  else
    return serpent.dump(tab, {name = "SaveData", comment = false, valtypeignore = {["function"] = true}})
  end
end

function deserialize(data)
  local ok, loaded_data = serpent.load(data)
  
  if ok then
    return loaded_data
  else
    err("Could not deserialize data!")
    return { }
  end
end

-- Check file exists

function exists(file)
  return love.filesystem.getInfo(file)
end

-- Table copier

function table.deepcopy(orig)
  local orig_type = type(orig)
  local copy
  if orig_type == 'table' then
    copy = {}
    for orig_key, orig_value in next, orig, nil do
      copy[table.deepcopy(orig_key)] = table.deepcopy(orig_value)
    end
    setmetatable(copy, table.deepcopy(getmetatable(orig)))
  else -- number, string, boolean, etc
    copy = orig
  end
  return copy
end

-- Merge multiple tables

function table.merge(...)
  local tabs = varg(...)
  
  local ret = { }
  for _, t in pairs(tabs) do
    for i, v in pairs(t) do
      ret[i] = v
    end
  end
  
  return ret
end

-- Append a table or two

function table.append(orig, ...)
  local tabs = varg(...)
  
  local ret = orig
  
  for _, add in ipairs(tabs) do
    if type(add) ~= "table" then
      table.insert(ret, add)
    else
      for i, v in ipairs(add) do
        table.insert(ret, v)
      end
    end
  end
  
  return ret
end

-- Table search functions

function table.has(tab, val)
  for k, v in pairs(tab) do
    if val == v then
      return true, k
    end
  end
end

function table.range(tab, start, finish)
  local ret = { }
  for i = start, finish do
    if tab[i] then
      table.insert(ret, tab[i])
    else
      break
    end
  end
  return ret
end

function table.equals(tab1, tab2)
  if #tab1 ~= #tab2 then
    return false
  end
  for k, v in ipairs(tab1) do
    if v ~= tab2[k] then
      return false
    end
  end
  return true
end

function table.random(tab)
  if rng then
    return tab[rng:random(1, #tab)]
  else
    return tab[love.math.random(1, #tab)]
  end
end

function table.weigh(tab)
  local weights = { }
  local total_weight = 0
  
  for k, v in pairs(tab) do
    total_weight = total_weight + v
    
    table.insert(weights, {total_weight + 0, k})
  end
  
  local weight = 0
  if rng then
    weight = rng:random(1, total_weight)
  else
    weight = love.math.random(1, total_weight)
  end
  
  for i, v in ipairs(weights) do
    if weight <= v[1] then
      return v[2]
    end
  end
end

-- Removes gametext specific table values and returns the base string

function stripString(tab)
  local ret = ""
  
  for i, v in ipairs(tab) do
    if type(v) == "string" and string.sub(v, 1, 2) ~= "/[" then
      ret = ret .. v
    end
  end
  
  return ret
end

-- Breaks multi-word strings into single words in a table

function breakString(str)
  local tab = { }
  
  for w in string.gmatch(str, "%S+") do
    table.insert(tab, w)
  end
  
  return tab
end

-- Get # of pairs

function table.num(tab)
  local ret = 0
  for i, t in pairs(tab) do
    ret = ret + 1
  end
  return ret
end

-- Get table made of keys

function table.ofKeys(tab)
  local ret = { }
  for k, _ in pairs(tab) do
    table.insert(ret, k)
  end
  return ret
end

-- Convert things to booleans

function tobool(thing)
  if type(thing) == "boolean" then
    return thing
  elseif type(thing) == "string" then
    if thing == "true" then
      return true
    elseif thing == "false" then
      return false
    else
      error("Expected string of \"true\" or \"false\", got \"" .. thing .. "\"")
    end
  elseif type(thing) == "number" then
    if thing == 1 then
      return true
    elseif thing == 0 then
      return false
    else
      error("Expected number of 1 or 0, got " .. thing)
    end
  else
    error("Expected string or number, got " .. type(thing))
  end
end

-- Convert other things to bool

function assess(val)
  if val == nil then
    return false
  end
  
  if type(val) == "boolean" then
    return val
  elseif type(val) == "function" then
    return val()
  else
    return tobool(val)
  end
end

-- Priority sort

function prioritySort(tab)
  table.sort(tab, prioritySorter)
end

function prioritySorter(a, b)
  if a.priority < b.priority then
    return true
  else
    return false
  end
end

-- Color format for positives and negatives

function posNeg(num, no_plus)
  if num > 0 then
    if no_plus then
      return {getColor("green"), tostring(num)}
    else
      return {getColor("green"), "+" .. tostring(num)}
    end
  elseif num < 0 then
    return {getColor("red"), tostring(num)}
  else
    return {tostring(num)}
  end
end

function percent(num)
  return tostring(math.round(num, 2) * 100) .. "%"
end

function percentPosNeg(num, bound_100, no_plus)
  local bound = ((bound_100 and 1) or 0)
  
  if num > bound then
    if no_plus then
      return {getColor("green"), percent(num)}
    else
      return {getColor("green"), "+" .. percent(num)}
    end
  elseif num < bound then
    return {getColor("red"), percent(num)}
  else
    return {percent(num)}
  end
end

function capitalize(str)
  return string.upper(string.sub(str, 1, 1)) .. string.sub(str, 2)
end

local tap = {
  {'A', 'B', 'C', 'D', 'E'},
  {'F', 'G', 'H', 'I', 'J'},
  {'L', 'M', 'N', 'O', 'P'},
  {'Q', 'R', 'S', 'T', 'U'},
  {'V', 'W', 'X', 'Y', 'Z'}
}

function toTapCode(str)
  local ret = ""
  str = string.gsub(string.upper(str), "K", "C")
  local first = true
  
  for i = 1, string.len(str) do
    local char = string.sub(str, i, i)
    
    for y = 1, #tap do
      for x = 1, #tap[y] do
        if char == tap[y][x] then
          ret = ret .. ((first and "") or " | ") .. string.format("%0" .. y .. "d", 0) .. " " .. string.format("%0" .. x .. "d", 0)
        end
      end
    end
    
    first = false
  end
  
  return ret
end

function toTapCodeTable(str)
  local ret = { }
  str = string.gsub(string.upper(str), "K", "C")
  
  for i = 1, string.len(str) do
    local char = string.sub(str, i, i)
    
    for y = 1, #tap do
      for x = 1, #tap[y] do
        if char == tap[y][x] then
          table.insert(ret, {y, x})
        end
      end
    end
  end
  
  return ret
end

-- HSL

function HSLtoRGB(h, s, l, a)
  local r, g, b
  
  if s == 0 then
    return l, l, l
  else
    local q, p
    if l < .5 then
      q = l * (1 + s)
    else
      q = l + s - l * s
    end
    p = 2 * l - q
    r = hueToRGB(p, q, h + (1/3))
    g = hueToRGB(p, q, h)
    b = hueToRGB(p, q, h - (1/3))
  end
  
  return r, g, b, a
end

function hueToRGB(p, q, t)
  if t < 0 then
    t = t + 1
  end
  if t > 1 then
    t = t - 1
  end
  if t < 1/6 then
    return p + (q - p) * 6 * t
  end
  if t < 1/2 then
    return q
  end
  if t < 2/3 then
    return p + (q - p) * ((2/3) - t) * 6
  end
  return p
end