-- Some debug assistance

local trace_text = { }
log_text = { }

local trace_hold = 4

-- Injects some stuff before the print function so we can put the text in our logs/trace text

local oldPrint = print

function print(...)
  local args = varg(...)
  
  for i, a in ipairs(args) do
    if type(a) == "table" then
      a = "TABLE"
    end
    
    table.insert(trace_text, {{0, .8, 1}, a, trace_hold, 1})
    table.insert(log_text, "[" .. os.date("%X") .. "] " .. a)
  end
  
  -- So print still does what it used to
  
  oldPrint(...)
end

function notify(...)
  local args = varg(...)
  
  for i, a in ipairs(args) do
    table.insert(trace_text, {{0, .8, 1}, a, trace_hold, 1})
  end
end

function soft_warn(...)
  local args = varg(...)
  
  for i, a in ipairs(args) do
    table.insert(trace_text, {{.8, .8, 0}, a, trace_hold, 1})
  end
end

function soft_err(...)
  local args = varg(...)
  
  for i, a in ipairs(args) do
    table.insert(trace_text, {{1, 0, 0}, a, trace_hold, 1})
  end
end

function log(...)
  if settings and settings.verbose then
    print(...)
    return
  end
  
  local args = varg(...)
  
  for i, a in ipairs(args) do
    table.insert(log_text, "[" .. os.date("%X") .. "] " .. a)
  end
  
  oldPrint(...)
end

function warn(...)
  local args = varg(...)
  
  for i, a in ipairs(args) do
    if settings.verbose then
      table.insert(trace_text, {{.8, .8, 0}, a, trace_hold, 1})
    end
    table.insert(log_text, "[" .. os.date("%X") .. "] WARN: " .. a)
    
    oldPrint("WARN: " .. a)
  end
end

function err(...)
  local args = varg(...)
  
  for i, a in ipairs(args) do
    table.insert(trace_text, {{1, 0, 0}, a, trace_hold, 1})
    table.insert(log_text, "[" .. os.date("%X") .. "] ERR: " .. a)
    
    oldPrint("ERR: " .. a)
  end
end

function dumpLog()
  return love.filesystem.write("log_" .. os.date("%Y-%m-%d_%H-%M-%S") .. ".txt", table.concat(log_text, "\r\n"))
end

function getLog()
  return log_text
end

function updateTrace(dt)
  local del = { }
  
  for i, t in ipairs(trace_text) do
    t[3] = t[3] - dt
    
    if t[3] <= 0 then
      t[4] = t[4] - dt
      
      if t[4] <= 0 then
        table.insert(del, i)
      end
    end
  end
  
  while #del > 0 do
    table.remove(trace_text, del[#del])
    table.remove(del)
  end
end

function drawTrace()
  for i, t in ipairs(trace_text) do
    local y = love.graphics.getHeight() - (system_font:getHeight() * (#trace_text - i + 1))
    if settings.top_down_trace then
      y = (system_font:getHeight() * (#trace_text - i))
    end
    
    love.graphics.setFont(system_font)
    love.graphics.setColor(t[1][1], t[1][2], t[1][3], t[4])
    love.graphics.rectangle("fill", 0, y, 2, system_font:getHeight())
    love.graphics.setColor(t[1][1], t[1][2], t[1][3], t[4] / 4)
    love.graphics.rectangle("fill", 0, y, love.graphics.getWidth(), system_font:getHeight())
    love.graphics.setColor(1, 1, 1, t[4])
    love.graphics.printo(t[2], 4, y)
  end
end